import unittest

from automon.integrations.instagram.client_browser import InstagramClientBrowser

c = InstagramClientBrowser()


class InstagramClientTest(unittest.TestCase):
    if c.authenticate():
        def test_authenticate(self):
            self.assertTrue(c.authenticate())


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):
    if c.isConnected():
        def test_soar_client_create_container_attachment(self):
            container = c.create_container(label='testing', name='testing')
            container = c.get_container(container_id=container.id)

            from .dino import dino
            attachment = c.create_container_attachment(
                container_id=container.id,
                file_name='dino.png',
                file_content=dino,
                metadata=None
            )

            self.assertTrue(attachment.vault_id)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.cryptocurrency import CryptoAccounting, Robinhood, Coinbase

with open('empty.csv', 'w') as f:
    f.write('''ASSET NAME,RECEIVED DATE,COST BASIS(USD),DATE SOLD,PROCEEDS''')

with open('robinhood.csv', 'w') as f:
    f.write('''Provider: Robinhood Crypto LLC,,,
Disclaimer: ,,,,
,,,,
,,,,
,,,,
,,,,
ASSET NAME,RECEIVED DATE,COST BASIS(USD),DATE SOLD,PROCEEDS''')


class CryptoAccountingTest(unittest.TestCase):
    c = CryptoAccounting()

    def test_CryptoAccounting(self):
        self.assertTrue(CryptoAccounting())

    def test_auto_detect(self):
        self.assertTrue(self.c.auto_detect('robinhood.csv'))
        self.assertFalse(self.c.auto_detect('empty.csv'))

    def test_walk(self):
        self.assertTrue(CryptoAccounting('.'))

    def test_robinhood(self):
        self.assertTrue(CryptoAccounting().robinhood('robinhood.csv'))

    def test_other(self):
        self.assertTrue(CryptoAccounting().other('empty.csv'))


class RobinhoodTest(unittest.TestCase):
    def test_Robinhood(self):
        self.assertTrue(Robinhood)


class CoinbaseTest(unittest.TestCase):
    def test_Coinbase(self):
        self.assertTrue(Coinbase)


if __name__ == '__main__':
    unittest.main()



import splunklib.binding as binding

from automon.log import Logging
from automon.helpers import environ

log = Logging(name='SplunkConfig', level=Logging.DEBUG)


class SplunkConfig:
    def __init__(self,
                 host: str = None,
                 port: int = None,
                 username: str = None,
                 password: str = None,
                 verify: str = True,
                 scheme: str = 'https',
                 app: NotImplemented = None,
                 owner: NotImplemented = None,
                 token: str = None,
                 cookie: str = None,
                 timeout: int = 1):
        """Splunk config"""

        self.host = host or environ('SPLUNK_HOST', 'splunkcloud.com')
        self.port = port or environ('SPLUNK_PORT', 8089)
        self.username = username or environ('SPLUNK_USERNAME', 'admin')
        self.password = password or environ('SPLUNK_PASSWORD', 'changeme')
        self.verify = verify
        self.scheme = scheme or 'https'
        self.app = app
        self.owner = owner
        self.token = token
        self.cookie = cookie
        self.handler = binding.handler(timeout=timeout)

    def info(self):
        return f'{self}'

    def __str__(self):
        return f'{self.__dict__}'



from automon.log import Logging
from automon.integrations.datascience import Pandas


class Robinhood:
    def __init__(self, csv):
        self._log = Logging(name=Robinhood.__name__, level=Logging.DEBUG)

        self.csv = None
        self.df = None
        self.is_match = None
        self.type = Robinhood.__name__

        r = RobinhoodCSV(csv)
        if r.matches:
            self._log.info(f'matched {r}')
            self.is_match = True
            self.csv = csv
            self.df = r.df

    def __repr__(self):
        return f'{self.csv}'

    def __eq__(self, other):
        if self.df.equals(other.df):
            return True
        return False


class RobinhoodCSV:
    def __init__(self, csv):
        """
        Provider: Robinhood Crypto LLC,,,
        Disclaimer: ,,,,
        ,,,,
        ,,,,
        ,,,,
        ,,,,
        ASSET NAME,RECEIVED DATE,COST BASIS(USD),DATE SOLD,PROCEEDS

        """
        self._log = Logging(name=RobinhoodCSV.__name__, level=Logging.DEBUG)

        self.csv = csv
        self.df = None
        self.matches = None

        if 'Provider: Robinhood Crypto LLC' in Pandas().read_csv(csv):
            self._log.debug(f'matched {csv}')
            self.matches = True

            with open(csv) as f:
                self.df = Pandas().csv_from_string(
                    ''.join(f.readlines()[6:])
                )

    def __repr__(self):
        return f'{self.df}'

    def __eq__(self, other):
        if isinstance(other, RobinhoodCSV):
            if self.df == other.df:
                self._log.debug(f'same {other}')
                return True
            self._log.debug(f'different {other}')
        return False



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):

    def test_update_playbook(self):
        if c.isConnected():
            container = c.create_container(label='testing', name='testing')
            playbook = ''

            if playbook:
                run = c.run_playbook(container_id=container.id, playbook_id=playbook)
                get_run = c.get_playbook_run(playbook_run_id=run.playbook_run_id)
                c.update_playbook(
                    playbook_id=get_run.playbook,
                    active=False
                )


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.elasticsearch.jvm import ElasticsearchJvmMonitor
from automon.integrations.elasticsearch.client import ElasticsearchClient


class ElasticsearchJvmMonitorTest(unittest.TestCase):

    def test_ElasticsearchJvmMonitor(self):
        self.assertTrue(ElasticsearchJvmMonitor)
        self.assertTrue(ElasticsearchJvmMonitor())
        self.assertFalse(ElasticsearchJvmMonitor().get_metrics())
        # self.assertFalse(ElasticsearchJvmMonitor().read_file())


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.google import PeopleClient

c = PeopleClient()


class TestClient(unittest.TestCase):
    def test_list_connections(self):
        if c.isConnected():
            self.assertTrue(list(c.list_connection_generator()))


if __name__ == '__main__':
    unittest.main()



from automon.log import Logging

from .datatypes import AbstractDataType

log = Logging(name='Asset', level=Logging.DEBUG)


class Asset(AbstractDataType):
    def __init__(self, asset: dict = {}):
        self.__dict__.update(asset)



import flask
from flask import Flask

from automon.log import Logging
from automon.integrations.flaskWrapper.config import FlaskConfig


class FlaskBoilerplate(object):

    def __init__(self, import_name: str = __name__, config: FlaskConfig = None, **kwargs):
        """Wrapper for flask"""
        self._log = Logging(FlaskBoilerplate.__name__, Logging.DEBUG)

        self.Flask = Flask(import_name=import_name, **kwargs)
        self.config = config or FlaskConfig()

    def __repr__(self):
        return f'{self.Flask}'

    @property
    def flask(self):
        return flask

    @property
    def request(self):
        """Get flask request"""
        return flask.request

    def enable_javascript_compatibility(self):
        """Enable Jinya compatibility for JavaScript"""
        self.Flask = FlaskConfig.javascript_compatibility(self.Flask)

    def run(self, port: int = None, debug: bool = False, **kwargs):
        """Run flask app"""
        return self.Flask.run(port=port, debug=debug, **kwargs)



import unittest

from automon.integrations.seleniumWrapper.browser import SeleniumBrowser

browser = SeleniumBrowser()
browser.set_driver(browser.type.chrome)


class SeleniumClientTest(unittest.TestCase):
    if browser.isRunning():
        def test(self):
            self.assertFalse(browser.get('http://555.555.555.555'))
            if browser.get('http://1.1.1.1'):
                self.assertTrue(True)

            if browser.get('http://google.com'):
                self.assertTrue(browser.get_screenshot_as_png())

            if browser.get('http://yahoo.com'):
                self.assertTrue(browser.get_screenshot_as_base64())

            if browser.get('http://bing.com'):
                self.assertTrue(browser.save_screenshot())
                self.assertTrue(browser.save_screenshot(folder='./'))

            browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):
    if c.isConnected():
        def test_create_artifact(self):
            id = c.create_container(label='testing', name='testing').id
            self.assertTrue(c.create_artifact(container_id=id))

        def test_create_container(self):
            self.assertTrue(c.create_container(label='testing', name='testing'))

        def test_delete_containers(self):
            container = c.create_container(label='testing', name='testing')
            self.assertTrue(c.delete_container(container_id=container.id))

        def test_get_container(self):
            container = c.create_container(label='testing', name='testing')
            self.assertTrue(c.get_container(container_id=container.id))

        def test_get_vault(self):
            container = c.create_container(label='testing', name='testing')

            list_vault = c.list_vault()
            vault = list_vault.get_one()
            if vault:
                self.assertTrue(c.get_vault(vault_id=vault.id))

        def test_isConnected(self):
            self.assertTrue(c.isConnected())

        def test_list_artifact(self):
            self.assertTrue(c.list_artifact())

        def test_list_containers(self):
            self.assertTrue(c.list_containers())

        def test_list_vault(self):
            self.assertTrue(c.list_vault())


if __name__ == '__main__':
    unittest.main()



def credential_db():
    """Hard coded credentials encrypted with bcrypt
    
    :return: credential database
    """

    # TODO: Encrypt user names
    # TODO: use an actual database

    credentials = {
        # 'admin': b'disabled',
        # 'test': b'disabled',
        'hev@hev': b'$2b$12$c.A//7LqYwWyEJXFOZ.ymuoJUwKfMJgl5zXGEbtaJ2gSCZBWtj/Hy',
    }

    return credentials



import unittest
import hashlib

from automon.integrations.minioWrapper import MinioClient
from automon.integrations.minioWrapper.bucket import Bucket

MINIO_ENDPOINT = 'play.minio.io:9000'
MINIO_ACCESS_KEY = 'Q3AM3UQ867SPQQA43P2F'
MINIO_SECRET_KEY = 'zuf+tfteSlswRu7BJ86wekitnifILbZam1KYY3TG'

client = MinioClient(endpoint=MINIO_ENDPOINT, access_key=MINIO_ACCESS_KEY, secret_key=MINIO_SECRET_KEY)
bucket = hashlib.md5(f'{MINIO_ENDPOINT}'.encode()).hexdigest()


class ClientTest(unittest.TestCase):

    def test_list_buckets(self):
        if client.isConnected():
            self.assertTrue(client.list_buckets())
            self.assertEqual(type(client.list_buckets()), list)

    def test_get_bucket(self):
        if client.isConnected():
            test = client.make_bucket(bucket)

            self.assertTrue(client.get_bucket(test))
            self.assertTrue(type(client.get_bucket(test)), Bucket)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):
    if c.isConnected():
        def test_list_vault_generator(self):
            test = [x for x in c.list_vault_generator(page_size=1, max_pages=1)]
            if test:
                self.assertTrue(test)


if __name__ == '__main__':
    unittest.main()



import json


class ServiceNow:
    table: str = None

    def __repr__(self):
        return f'{self.__dict__}'

    @property
    def fields(self):
        return self.to_json()

    def to_dict(self):
        return {k: v for k, v in self.__dict__.items() if v}

    def to_json(self):
        return json.dumps(self.to_dict())



import os
import io

from automon.integrations.minioWrapper import MinioClient, MinioConfig
from automon.log import Logging
from automon.helpers.sleeper import Sleeper

log = Logging(name='openvpn', level='info')


class ClientConfig:
    """ Create an OpenVPN client config
    """

    def __init__(self, name, hosts, options=None):
        """
        client
        dev tun
        remote example.com
        resolv-retry infinite
        nobind
        persist-key
        persist-tun
        ca [inline]
        cert [inline]
        key [inline]
        tls-auth [inline] 1
        verb 1
        keepalive 10 120
        port 1194
        proto udp
        cipher BF-CBC
        comp-lzo
        remote-cert-tls server

        <ca>
        </ca>

        <cert>
        </cert>

        <key>
        </key>

        <tls-auth>
        </tls-auth>
        """

        log.logging.debug('[ClientConfig] Creating new OpenVPN client config')

        self.name = name

        self.config = 'client\n'
        self.config += 'dev tun\n'

        for host in hosts:
            host, port, conn = host.split(':')
            self.config += f'remote {host} {port} {conn}\n'

        self.config += 'resolv-retry infinite\n'
        self.config += 'nobind\n'
        self.config += 'persist-key\n'
        self.config += 'persist-tun\n'
        self.config += 'ca [inline]\n'
        self.config += 'cert [inline]\n'
        self.config += 'key [inline]\n'
        self.config += 'tls-auth [inline] 1\n'
        self.config += 'verb 1\n'
        self.config += 'keepalive 10 120\n'
        self.config += 'cipher BF-CBC\n'
        self.config += 'comp-lzo\n'
        self.config += 'remote-cert-tls server\n'
        self.config += 'key-direction 1\n'

        if options:
            for option in options:
                self.config += option

        self.CONFIG = self.config.split('\n')

        self.ca = None
        self.cert = None
        self.key = None
        self.ta = None

    def add_ca(self, ca):
        self.ca = f'<ca>\n{ca.decode()}\n</ca>\n'

    def add_cert(self, cert):
        self.cert = f'<cert>\n{cert.decode()}\n</cert>\n'

    def add_key(self, key):
        self.key = f'<key>\n{key.decode()}\n</key>\n'

    def add_ta(self, ta):
        self.ta = f'<tls-auth>\n{ta.decode()}\n</tls-auth>\n'

    def build_config(self, prefix):

        config = self.config
        config += self.ca
        config += self.cert
        config += self.key
        config += self.ta

        if prefix:
            filename = prefix + '-' + self.name + '.ovpn'
        else:
            filename = self.name + '.ovpn'

        return filename, io.BytesIO(config.encode()), len(config)


def collector(minio_client, bucket, folder):
    """ Collect required files to build an OpenVPN client
    """
    log.logging.debug('[collector] Collecting all Minio bucket files')

    ca = None
    cert = []
    key = []
    ta = None

    for file in minio_client.list_objects(bucket, folder):

        file_path, file_name = os.path.split(file.object_name)
        folder = os.path.split(file_path)[-1]

        if file_name == 'ca.crt':
            download = minio_client.download_object(bucket, file)
            ca = download.data

        if file_name == 'ta.key':
            download = minio_client.download_object(bucket, file)
            ta = download.data

        if folder == 'issued':
            download = minio_client.download_object(bucket, file)
            data = download.data
            cert.append((file_name, data))

        if folder == 'private':
            download = minio_client.download_object(bucket, file)
            data = download.data
            key.append((file_name, data))

    return ca, cert, key, ta


def put_object(minio_client: MinioClient, bucket, client_configs, config_name, config_data, config_len):
    """ Minio object uploader
    """
    log.logging.debug(f'[put_object] Uploading: {config_name}')
    return minio_client.put_object(bucket, f'{client_configs}/{config_name}', config_data, config_len)


def create_configs(minio_client: MinioClient, bucket: str, client_configs: str,
                   ca: bytes, cert: list, key: list, ta: bytes, hosts: str, prefix, options=None):
    """ Create and upload OpenVPN Client config
    """
    for user, data in cert:
        name, _ = os.path.splitext(user)
        config = ClientConfig(name, hosts, options)
        config.add_ca(ca)
        config.add_ta(ta)
        config.add_cert(data)

        for k in key:
            k_name, k_data = k
            k_name, _ = os.path.splitext(k_name)

            if name == k_name:
                config.add_key(k_data)

        config_name, config_data, config_len = config.build_config(prefix)

        put_object(minio_client, bucket, client_configs, config_name, config_data, config_len)

        log.logging.debug(f'[create_configs] OpenVPN client config uploaded: {config_name}')


def run(minio_config: MinioConfig, openvpn_config):
    log.logging.info('Running...')

    while True:
        minio_config.secure = False
        minio_client = MinioClient(config=minio_config)

        minio_bucket = openvpn_config['bucket']
        openvpn_configs = openvpn_config['configs']

        for config in openvpn_configs:
            hosts = config['hosts']
            folder = os.path.relpath(config['folder'])
            keys = os.path.join(folder, 'pki')
            client_configs = os.path.join(folder, 'configs')
            try:
                prefix = config['prefix']
            except:
                prefix = None
            try:
                options = config['options']
            except:
                options = None

            ca, cert, key, ta = collector(minio_client, minio_bucket, keys)
            create_configs(minio_client, minio_bucket, client_configs, ca, cert, key, ta, hosts, prefix, options)

        log.logging.info('[build client configs] Finshed building all OpenVPN clients')
        log.logging.debug('[ClientConfig] sleeping')
        Sleeper.day('openvpn')


def test_run(minio_config: MinioConfig, openvpn_config):
    while True:
        minio_config.secure = False
        minio_client = MinioClient(minio_config)

        minio_bucket = openvpn_config['bucket']
        openvpn_configs = openvpn_config['configs']

        for config in openvpn_configs:
            hosts = config['hosts']
            folder = os.path.relpath(config['folder'])
            keys = os.path.join(folder, 'pki')
            client_configs = os.path.join(folder, 'configs')
            try:
                prefix = config['prefix']
            except:
                prefix = None
            try:
                options = config['options']
            except:
                options = None

            ca, cert, key, ta = collector(minio_client, minio_bucket, keys)
            create_configs(minio_client, minio_bucket, client_configs, ca, cert, key, ta, hosts, prefix, options)

            return True



import unittest

from automon.integrations.neo4jWrapper.client import Neo4jClient
from automon.integrations.neo4jWrapper.cypher import Cypher


class Neo4jTest(unittest.TestCase):
    client = Neo4jClient()

    def test_build_cypher(self):
        c = Neo4jClient()
        c.build_cypher('AAAA')
        self.assertTrue(c.cypher)
        self.assertEqual(c.cypher, 'AAAA')

    def test_create_dict(self):

        if self.client.isConnected():
            self.assertTrue(self.client.create_dict(prop='name', value='finn', label='human', data={'powers': 'no'}))
            self.assertTrue(self.client.create_dict(prop='name', value='jake', label='dog', data={'powers': True}))

    def test_merge(self):

        if self.client.isConnected():
            self.assertTrue(self.client.merge(label='dog'))

    def test_merge_dict(self):

        if self.client.isConnected():
            self.assertTrue(self.client.merge_dict(
                prop='name', value='lsp', label='lumpy', data={'powers': 'princess'}))
            self.assertTrue(self.client.merge_dict(
                prop='name', value='bubble gum', label='bubble',
                data={'powers': 'princess', 'specialty': 'science'}))
            self.assertTrue(self.client.merge_dict(
                prop='name', value='BMO', label='gameboy',
                data={'powers': 'buttons', 'specialty': ['green', 'box']}))

    def test_relationships(self):
        if self.client.isConnected():
            self.assertTrue(self.client.merge_dict(
                prop='name', value='lsp', label='lumpy', data={'powers': 'princess'}))
            self.assertTrue(self.client.merge_dict(
                prop='name', value='bubble gum', label='bubble',
                data={'powers': 'princess', 'specialty': 'science'}))
            self.assertTrue(self.client.merge_dict(
                prop='name', value='BMO', label='gameboy',
                data={'powers': 'buttons', 'specialty': ['green', 'box']}))

            self.assertTrue(self.client.relationship(
                A_prop='powers', A_value='princess',
                B_prop='type', B_value='princess', B_label='princess',
                label='is',
                direction='->'
            ))

    def test_delete_node(self):
        if self.client.isConnected():
            self.assertTrue(self.client.merge_dict(
                prop='name', value='Marceline', label='vampire', data={'age': '1000'}))
            self.assertTrue(self.client.delete_match(prop='name', value='Marceline'))

    def test_assert_label(self):
        test = Cypher()

        self.assertEqual(test.assert_label(':test'), ':`test`')
        self.assertIsNone(test.assert_label('1test'))
        self.assertEqual(test.assert_label('works'), ':`works`')


if __name__ == '__main__':
    unittest.main()



import minio


class Object(minio.datatypes.Object):
    bucket_name: str
    object_name: str

    def __init__(self, object: minio.datatypes.Object):
        self.__dict__.update(object.__dict__)

    def __repr__(self):
        return self.object_name


class DeleteObject(minio.deleteobjects.DeleteObject):
    name: str
    version_id: str

    def __init__(self, object: minio.deleteobjects.DeleteObject):
        self.__dict__.update(object.__dict__)
        self._name = object.object_name

    def __repr__(self):
        return self.name

    @property
    def bucket_name(self):
        return self._bucket_name

    @property
    def name(self):
        return self._name

    @property
    def version_id(self):
        return self._version_id



import functools
from queue import Queue

import splunklib.results
import splunklib.client

from automon.log import Logging
from automon.integrations.splunk.config import SplunkConfig
from automon.integrations.splunk.helpers import Job, Application

log = Logging(name='SplunkClient', level=Logging.DEBUG)


class SplunkRestClient:

    def __int__(self, config: SplunkConfig = SplunkConfig()):
        self.config = config


class SplunkClient(object):

    def __init__(self, config: SplunkConfig = None):
        """Splunk client"""

        self.config = config or SplunkConfig()
        self.queue = Queue()

    def __str__(self):
        if self.is_connected():
            return f'connected to {self.config}'
        return f'not connected to {self.config}'

    @property
    def client(self) -> splunklib.client.connect:
        return splunklib.client.connect(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password,
            verify=self.config.verify,
            scheme=self.config.scheme,
            app=self.config.app,
            owner=self.config.owner,
            token=self.config.token,
            cookie=self.config.cookie,
            handler=self.config.handler
        )

    @property
    def service(self) -> splunklib.client.connect:
        return self.client()

    def _is_connected(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            try:
                self.client
                return func(self, *args, **kwargs)
            except Exception as e:
                log.error(f'not connected. {e}', enable_traceback=False)
            return False

        return wrapped

    @_is_connected
    def is_connected(self) -> bool:
        return True

    def info(self):
        return f'{self}'

    def search(self, query) -> list:
        """create normal search query"""
        kwargs_normalsearch = {"exec_mode": "normal"}
        job = self.create_job(query, **kwargs_normalsearch)
        # job = self.create_job(query)
        return self.results(job)

    def oneshot(self, query, earliest_time: str = '-15m', latest_time: str = 'now'):
        """create oneshot search"""

        kwargs_oneshot = {
            "earliest_time": earliest_time,
            "latest_time": latest_time
        }

        searchquery_oneshot = query
        oneshotsearch_results = self.service.jobs.oneshot(searchquery_oneshot, **kwargs_oneshot)

        reader = splunklib.results.ResultsReader(oneshotsearch_results)

        return [x for x in reader]

    def query(self, query):
        return self.search(query)

    def jobs(self):
        """get collection of search jobs"""
        return self.service.jobs

    def create_job(self, query, **kwargs):
        """create search job"""
        return self.service.jobs.create(query, **kwargs)

    @staticmethod
    def results(job: splunklib.client.Job) -> list:
        """io blocking waiting for job results"""
        j = Job(job)
        while True:
            while not j.is_ready():
                pass
            if j.isDone == '1':
                break

        results = [x for x in splunklib.results.ResultsReader(j.results())]
        return results

    def job_summary(self):
        return f'There are {len(self.jobs())} jobs available to the current user'

    def get_apps(self) -> [Application]:
        return [Application(x) for x in self.service.apps]

    def create_app(self, app_name):
        return self.service.apps.create(app_name)

    def get_app(self, app_name):
        return self.service.apps[app_name]

    def delete_app(self, app_name):
        return self.service.apps.delete(app_name)

    def app_info(self, app_name):
        return f'{self.service.apps[app_name]}'



import unittest

from automon.integrations.minioWrapper.config import MinioConfig

c = MinioConfig()


class ConfigTest(unittest.TestCase):

    def test_MinioConfig(self):
        if c.isReady():
            self.assertTrue(MinioConfig().isReady())
        else:
            self.assertFalse(MinioConfig().isReady())


if __name__ == '__main__':
    unittest.main()



import splunklib.client as client

from automon.log import Logging

log = Logging(name=__name__, level=Logging.DEBUG)


class JobError:

    def __init__(self, job: client.Job):
        self._log = Logging(JobError.__name__, level=Logging.DEBUG)
        self._job = job

        self.messages = job.content['messages']
        try:
            self.fatal = self.messages['fatal']
            self.error = self.messages['error']

            self._log.critical(self.fatal)
            self._log.error(self.error)
        except Exception as e:
            self._log.error(e)


class Job:

    def __init__(self, job: client.Job):
        self._log = Logging(Job.__name__, level=Logging.DEBUG)
        self._job = job

        try:
            self.sid = job['sid']
            self.isDone = job["isDone"]
            self.doneProgress = float(job["doneProgress"]) * 100
            self.scanCount = int(job["scanCount"])
            self.eventCount = int(job["eventCount"])
            self.resultCount = int(job["resultCount"])
            self._searchProviders = job.content['searchProviders']
            self._search = job.content['search'] if 'search' in job.content.keys() else None
            self._runDuration = job.content['runDuration'] if 'runDuration' in job.content.keys() else None
            self._pid = job.content['pid'] if 'pid' in job.content.keys() else None
            self._normalizedSearch = job.content[
                'normalizedSearch'] if 'normalizedSearch' in job.content.keys() else None
            self._earliestTime = job.content['earliestTime']
            self._cursorTime = job.content['cursorTime']
            self._diskUsage = job.content['diskUsage']

            self.error = JobError(job)
        except Exception as e:
            self._log.error(e)

    def is_ready(self):
        return self._job.is_ready()

    def results(self):
        return self._job.results()

    def access(self):
        return self._job.access

    def content(self):
        return self._job.content

    def __str__(self):
        return f'{self._search} {self._searchProviders} ({self._runDuration} s) ({self._diskUsage} B)'


class Application:
    def __init__(self, app: dict):
        self._app = app

        self.access = app['access']
        self.content = app['content']
        self.defaults = app['defaults']
        self.fields = app['fields']
        self.links = app['links']
        self.name = app['name']
        self.path = app['path']
        self.service = app['service']
        self.setupInfo = app['setupInfo']
        self.state = app['state']

        self._state = app['_state']



from automon.log import Logging
from ..config import SplunkSoarConfig

config = SplunkSoarConfig()
log = Logging(name='Urls', level=Logging.DEBUG)


class Urls:
    """REST API endpoints

    /rest/<type>/[identifier]/[detail]?page=0&page_size=10&pretty&_filter_XXX='YYY'&_exclude_XXX='YYY'&include_expensive
    """
    HOST = config.host
    REST = f'{HOST}/rest'

    # type
    ACTION_RUN = f'{REST}/action_run'
    APP = f'{REST}/app'
    APP_RUN = f'{REST}/app_run'
    ARTIFACT = f'{REST}/artifact'
    ASSET = f'{REST}/asset'
    CLUSTER_NODE = f'{REST}/cluster_node'
    CONTAINER = f'{REST}/container'
    CONTAINER_ATTACHMENT = f'{REST}/container_attachment'
    PLAYBOOK = f'{REST}/playbook'
    PLAYBOOK_RUN = f'{REST}/playbook_run'
    VAULT = f'{REST}/vault_document'
    VAULT_ADD = f'{REST}/vault_add'
    VAULT_DELETE = f'{REST}/vault_delete'
    VAULT_INFO = f'{REST}/vault_info'

    GENERIC = f'{REST}/'

    @classmethod
    def generic(cls, api: str, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.GENERIC}{api}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def action_run(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.ACTION_RUN}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def app(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.APP}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def app_run(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.APP_RUN}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def artifact(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.ARTIFACT}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def asset(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.ASSET}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def container(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.CONTAINER}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def container_attachment(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.CONTAINER_ATTACHMENT}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def cluster_node(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.CLUSTER_NODE}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def exclude(cls, field_name: str, value: str or int or list or None):
        """Excluding

        docs: https://docs.splunk.com/Documentation/SOAR/current/PlatformAPI/RESTQueryData#Excluding

        Examples:
            /rest/endpoint?_exclude_<field_name>=<value>
            /rest/artifact?_exclude_tags__contains="preprocessed"
        """

        if isinstance(value, str):
            value = f'"{value}"'

        elif isinstance(value, int):
            value = int({value})

        elif isinstance(value, list):
            new_value = []

            for v in value:
                if isinstance(v, str):
                    new_value.append(f'"{v}"')
                elif isinstance(v, int):
                    new_value.append(f'{v}')
                else:
                    new_value.append(v)

            value = new_value

        return f'_exclude_{field_name}={value}'

    @classmethod
    def filter(cls, field_name: str, value: str or int or list or None):
        """Filtering

        docs: https://docs.splunk.com/Documentation/SOAR/current/PlatformAPI/RESTQueryData#Filtering

        Examples:
            /rest/endpoint?_filter_<field_name>=<value>

            &_filter_type="network"

        Type	Format	Example
        string	Enclosed in double or single quotes.	"myvalue"
        number	no formatting	10
        array	Python syntax for a literal list.	["a string", 2, None]
        """

        if isinstance(value, str):
            value = f'"{value}"'

        elif isinstance(value, int):
            value = int({value})

        elif isinstance(value, list):
            new_value = []

            for v in value:
                if isinstance(v, str):
                    new_value.append(f'"{v}"')
                elif isinstance(v, int):
                    new_value.append(f'{v}')
                else:
                    new_value.append(v)

            value = new_value

        return f'_filter_{field_name}={value}'

    @classmethod
    def playbook(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.PLAYBOOK}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def playbook_run(cls, identifier: int = None, detail: str = None, *args, **kwargs):
        return f'{cls.PLAYBOOK_RUN}{cls.query(identifier=identifier, detail=detail, *args, **kwargs)}'

    @classmethod
    def params(cls, page: int = None,
               page_size: int = None,
               pretty: bool = None,
               filter: (str, str or int or list or None) = None,
               exclude: str = None,
               include_expensive: bool = None,
               sort: str = None,
               order: str = None, **kwargs) -> str:

        """General Form for a Parameters

        docs: https://docs.splunk.com/Documentation/SOAR/current/PlatformAPI/RESTQueryData#General_Form_for_a_Query

        ?page=0&page_size=10&pretty&_filter_XXX='YYY'&_exclude_XXX='YYY'&include_expensive

        Parameter	Required	Description
        type	required	The type of data being queried. Supported types are:
        action_run
        artifact
        asset
        app
        app_run
        container
        playbook_run
        cluster_node
        Querying for only a type will return a paginated array of objects.

        identifier	optional	Adding an identifier to a URL will retrieve a single specific object. Identifiers are integers and are unique for each resource.
        detail	optional	Adding a detail can only be done when querying a single object. The result is to return information on a single field of the object.
        page	optional	Positive integer. Returned results are paginated. This query parameter requests a specific page.
        page_size	optional	Positive integer. Returned results are paginated. This query parameter determines how many results returned per-page. Use "0" for all results.
        Setting page_size to 0 will return all results. Querying a very large data set can have an adverse effect on performance.

        pretty	optional	No value. Adding ?pretty (or &pretty) to your query will add related or calculated fields prefixed with _pretty_ to the response. For instance if the record you are looking for has a start_time, the response will have _pretty_start time that is a relative local version of the time. A record that has an "owner" reporting back an id will gain _pretty_owner that shows the owner's display name.
        Requesting pretty values is a relatively expensive call since it may involve expensive calculations or additional database lookups. When querying individual records or small numbers of records, this should not cause any performance hit. However if requesting tens of thousands of records it may have an adverse impact on your system depending on your hardware.

        _filter_XXX	optional	Add one or more filters to limit the results. Applies only to lists of objects. See Filtering below.
        _exclude_XXX	optional	Exclude matching items with syntax similar to filtering. See Excluding below.
        include_expensive	optional	No value. Adding this flag will cause the REST API to return all fields when returning a list, including large/expensive fields.
        The include_expensive parameter will return all fields, just as if you were requesting the individual record. These expensive fields may have megabytes of data for a single record, so use this option with caution as it may have a significant performance impact if returning large amounts of data.

        sort	optional	Field to sort results with. Can be any "simple" field at the top-level of record, such as a string, boolean, or integer value that is not under a hierarchy. Custom fields for events can also be sorted, using the format custom_fields.field_name.
        order	string	Either "asc" or "desc". This is the sorting order for the results.
        """

        params = []

        if page is not None:
            params.append(f'page={page}')

        if page_size is not None:
            params.append(f'page_size={page_size}')

        if pretty:
            params.append(f'pretty')

        if filter:
            field_name, value = filter
            params.append(f'{cls.filter(field_name=field_name, value=value)}')

        if exclude:
            params.append(f'{cls.exclude(exclude)}')

        if include_expensive:
            params.append(f'{include_expensive}')

        if sort:
            """field to sort by"""
            params.append(f'{sort}')

        if order:
            """asc or desc"""
            params.append(f'{order}')

        if params:
            params = f'?{"&".join(params)}'

        return params

    @classmethod
    def query(cls,
              identifier: int = None,
              detail: str = None,
              page: int = None,
              page_size: int = None,
              pretty: bool = None,
              filter: (str, str or int or list or None) = None,
              exclude: str = None,
              include_expensive: bool = None,
              sort: str = None,
              order: str = None, **kwargs):
        """General Form for a Query

        docs: https://docs.splunk.com/Documentation/SOAR/current/PlatformAPI/RESTQueryData#General_Form_for_a_Query

        ?page=0&page_size=10&pretty&_filter_XXX='YYY'&_exclude_XXX='YYY'&include_expensive

        Parameter	Required	Description
        type	required	The type of data being queried. Supported types are:
        action_run
        artifact
        asset
        app
        app_run
        container
        playbook_run
        cluster_node
        Querying for only a type will return a paginated array of objects.

        identifier	optional	Adding an identifier to a URL will retrieve a single specific object. Identifiers are integers and are unique for each resource.
        detail	optional	Adding a detail can only be done when querying a single object. The result is to return information on a single field of the object.
        page	optional	Positive integer. Returned results are paginated. This query parameter requests a specific page.
        page_size	optional	Positive integer. Returned results are paginated. This query parameter determines how many results returned per-page. Use "0" for all results.
        Setting page_size to 0 will return all results. Querying a very large data set can have an adverse effect on performance.

        pretty	optional	No value. Adding ?pretty (or &pretty) to your query will add related or calculated fields prefixed with _pretty_ to the response. For instance if the record you are looking for has a start_time, the response will have _pretty_start time that is a relative local version of the time. A record that has an "owner" reporting back an id will gain _pretty_owner that shows the owner's display name.
        Requesting pretty values is a relatively expensive call since it may involve expensive calculations or additional database lookups. When querying individual records or small numbers of records, this should not cause any performance hit. However if requesting tens of thousands of records it may have an adverse impact on your system depending on your hardware.

        _filter_XXX	optional	Add one or more filters to limit the results. Applies only to lists of objects. See Filtering below.
        _exclude_XXX	optional	Exclude matching items with syntax similar to filtering. See Excluding below.
        include_expensive	optional	No value. Adding this flag will cause the REST API to return all fields when returning a list, including large/expensive fields.
        The include_expensive parameter will return all fields, just as if you were requesting the individual record. These expensive fields may have megabytes of data for a single record, so use this option with caution as it may have a significant performance impact if returning large amounts of data.

        sort	optional	Field to sort results with. Can be any "simple" field at the top-level of record, such as a string, boolean, or integer value that is not under a hierarchy. Custom fields for events can also be sorted, using the format custom_fields.field_name.
        order	string	Either "asc" or "desc". This is the sorting order for the results.
        """
        query = ''
        params = cls.params(
            page=page,
            page_size=page_size,
            pretty=pretty,
            filter=filter,
            exclude=exclude,
            include_expensive=include_expensive,
            sort=sort,
            order=order, **kwargs
        )

        if identifier is not None:
            query += f'/{identifier}'

        if detail is not None:
            query += f'/{detail}'

        if params:
            query += f'{params}'

        return query

    @classmethod
    def vault(cls, identifier: int = None, *args, **kwargs):
        return f'{cls.VAULT}{cls.query(identifier=identifier, *args, **kwargs)}'

    @classmethod
    def vault_add(cls, identifier: int = None, *args, **kwargs):
        return f'{cls.VAULT_ADD}{cls.query(identifier=identifier, *args, **kwargs)}'

    @classmethod
    def vault_delete(cls, identifier: int = None, *args, **kwargs):
        return f'{cls.VAULT_DELETE}{cls.query(identifier=identifier, *args, **kwargs)}'

    @classmethod
    def vault_info(cls, identifier: int = None, *args, **kwargs):
        return f'{cls.VAULT_INFO}{cls.query(identifier=identifier, *args, **kwargs)}'



import unittest

from automon.integrations.splunk import SplunkClient

cloud = SplunkClient()


class SplunkClientTest(unittest.TestCase):

    def test_client_connect(self):
        if cloud.is_connected():
            self.assertIsNotNone(cloud)
            self.assertIsNotNone(cloud.client)
            self.assertTrue(cloud.get_apps())
            self.assertTrue([x.name for x in cloud.get_apps()])

    def test_init(self):
        self.assertIsNotNone(SplunkClientTest())

    def test_jobs(self):
        if cloud.is_connected():
            self.assertTrue(cloud.jobs())
            self.assertTrue(cloud.job_summary())
            self.assertTrue(cloud.create_job('search * '))

    def test_search(self):
        if cloud.is_connected():
            self.assertTrue(cloud.search('search index=* | head 10'))

    def test_oneshot(self):
        if cloud.is_connected():
            self.assertIsNotNone(cloud.oneshot('search * | head 10'))


if __name__ == '__main__':
    unittest.main()



class SwiftError_(object):
    def __init__(self, error: dict):
        self.action = error.get('action', '')
        self.container = error.get('container', '')
        self.headers = error.get('headers', '')
        self.success = error.get('success', '')
        self.error = error.get('error', '')
        self.traceback = error.get('traceback', '')
        self._tb = self.traceback
        self.error_timestamp = error.get('error_timestamp', '')
        self.response_dict = error.get('response_dict', '')

    def __str__(self):
        return (
            f'Error:\n'
            f'>*Action*: {self.action}\n'
            f'>*Container*: {self.container}\n'
            f'>*Error*: {self.error}\n'
            f'Traceback: \n```{self.traceback}```'
        )



class XPaths(object):

    def __repr__(self):
        return 'Instagram XPaths'

    @property
    def login_user(self):
        return '//*[@id="loginForm"]/div/div[1]/div/label/input'

    @property
    def login_pass(self):
        return '//*[@id="loginForm"]/div/div[2]/div/label/input'

    @property
    def login_button(self):
        return '//*[@id="loginForm"]/div/div[3]/button'

    @property
    def login_pass_xpaths(self):
        return [
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[3]/div/label/input',
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[4]/div/label/input'
        ]

    @property
    def login_btn_xpaths(self):
        return [
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[4]/button',
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[6]/button'
        ]

    @property
    def save_your_login_info(self):
        return '//*[@id="react-root"]/section/main/div/div/div/section/div/button'


from datetime import datetime, date


class Dates:
    @staticmethod
    def today():
        return datetime.today()

    @staticmethod
    def now():
        return datetime.now()

    @staticmethod
    def iso():
        return datetime.isoformat(datetime.now())

    @staticmethod
    def iso_short():
        return date.today().isoformat()

    @staticmethod
    def filename_timestamp():
        return f'{datetime.now().isoformat()}'.replace(':', '_')



import unittest

from automon.helpers.subprocessWrapper import Run

run = Run()


class TestRun(unittest.TestCase):
    def test_false(self):
        self.assertFalse(run.run(''))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.elasticsearch.client import ElasticsearchClient
from automon.integrations.elasticsearch.config import ElasticsearchConfig


class ClientTest(unittest.TestCase):
    e = ElasticsearchClient()

    def test_create_document(self):
        es = ElasticsearchClient()

        if es.isConnected():
            self.assertTrue(es.isConnected())


if __name__ == '__main__':
    unittest.main()



from automon import Logging

from automon.helpers.osWrapper.environ import environ

log = Logging('InstagramConfig', level=Logging.INFO)


class InstagramConfig(object):

    def __init__(self, login: str = None, password: str = None):
        self.login = login or environ('INSTAGRAM_LOGIN', '')
        self.password = password or environ('INSTAGRAM_PASSWORD', '')

    def isConfigured(self):
        if self.login and self.password:
            log.info(f'OK')
            return True
        log.warn(f'BAD')
        return False

    def __repr__(self):
        if self.isConfigured():
            return f'ready'
        return f'not ready'



class Format:
    blockquote = '>'
    codeblock = '```'


class Chat:
    @staticmethod
    def Format(msg: str, msg_format: Format):
        return f'{msg_format}{msg}'

    @staticmethod
    def wrap(msg: str, msg_format: Format):
        return f'{msg_format}{msg}{msg_format}'

    @staticmethod
    def none():
        return ''

    @staticmethod
    def clean(msg: str or None) -> str or None:
        if msg is None:
            return None
        return f'{msg}'.strip()

    @staticmethod
    def string(msg: str or None) -> str or None:
        if msg is None:
            return ''
        return f'{msg}'

    @staticmethod
    def multi_to_single_line(msg: str) -> str:
        if msg is None:
            return f''

        if '\n' in msg and msg is not None:

            new_msg = []
            split = msg.splitlines()

            for line in split:
                if line:
                    line = line.strip()
                    line = Chat.clean(line)
                    new_msg.append(line)

            new_msg = ' '.join(new_msg)

            return new_msg

        return msg



from automon.log import Logging
from automon.helpers import Run


class NmapConfig(object):
    def __init__(self, **kwargs):
        self._log = Logging(name=NmapConfig.__name__, level=Logging.ERROR)

        self.nmap = None

    def __repr__(self):
        if self.isReady():
            return f'{self.nmap}'
        return f'nmap not found'

    def isReady(self, **kwargs):
        check = Run('which nmap', **kwargs)

        if check.stdout:
            self.nmap = check.stdout.decode().strip()
            self._log.debug(f'nmap located, {self.nmap}')
            return True
        else:
            self._log.error(f'nmap not found', enable_traceback=False)

        return False



import selenium

from automon.log import Logging

from .config import SeleniumConfig

log = Logging(name='BrowserType', level=Logging.DEBUG)


class BrowserType(object):

    def __init__(self, config: SeleniumConfig):
        self.config = config
        self.webdriver = self.config.webdriver
        self.chromedriver = self.config.selenium_chromedriver_path

    def __repr__(self):
        return 'BrowserType'

    @property
    def chrome(self):
        log.info(f'Browser set as Chrome')
        try:
            if self.chromedriver:
                return self.webdriver.Chrome(self.chromedriver)
            return self.webdriver.Chrome()
        except Exception as e:
            log.error(f'Browser not set. {e}', enable_traceback=False)

    @property
    def chromium_edge(self):
        log.info(f'Browser set as Chromium Edge')
        try:
            if self.chromedriver:
                return self.webdriver.ChromiumEdge(self.chromedriver)
            return self.webdriver.ChromiumEdge()
        except Exception as e:
            log.error(f'Browser not set. {e}', enable_traceback=False)

    @property
    def edge(self):
        log.info(f'Browser set as Edge')
        return self.webdriver.Edge()

    @property
    def firefox(self):
        log.info(f'Browser set as Firefox')
        return self.webdriver.Firefox()

    @property
    def ie(self):
        log.info(f'Browser set as Internet Explorer')
        return self.webdriver.Ie()

    @property
    def opera(self):
        log.info(f'Browser set as Opera')
        return self.webdriver.Opera()

    @property
    def proxy(self):
        log.info(f'Browser using proxy')
        return self.webdriver.Proxy()

    @property
    def remote(self):
        log.info(f'Browser using remote browser')
        return self.webdriver.Remote()

    @property
    def safari(self):
        log.info(f'Browser set as Safari')
        return self.webdriver.Safari()

    @property
    def webkit_gtk(self):
        log.info(f'Browser set as WebKitGTK')
        return self.webdriver.WebKitGTK()

    @property
    def wpewebkit(self):
        log.info(f'Browser set as WPEWebKit')
        return self.webdriver.WPEWebKit()



from automon.log import Logging


class SlackError:
    def __init__(self, error: Exception):
        """The request to the Slack API failed.
        The server responded with:
        {
            "ok": false,
            "error": "missing_scope",
            "needed": "users:read",
            "provided": "chat:write,chat:write.customize,chat:write.public,links:write,links:read,files:read,files:write"
        }
        """

        self._log = Logging(name=SlackError.__name__, level=Logging.ERROR)

        self._error = error
        self._reason = getattr(self._error, 'reason', '')
        self._response = getattr(self._error, 'response', '')

        if self._reason:
            self.args = getattr(self._reason, 'args', '')
            self.errno = getattr(self._reason, 'errno', '')
            self.reason = getattr(self._reason, 'reason', '')
            self.strerror = getattr(self._reason, 'strerror', '')
            self.verify_code = getattr(self._reason, 'verify_code', '')
            self.verify_message = getattr(self._reason, 'verify_message', '')

        if self._response:
            self.api_url = getattr(self._response, 'api_url' '')

            self.data = dict(getattr(self._response, 'data', ''))
            self.ok = self.data.get('ok', '')
            self.__error = self.data.get('error', '')
            self._needed = self.data.get('needed', '')
            self.provided = self.data.get('provided', '')

            self.headers = getattr(self._response, 'headers', '')
            self.http_verb = getattr(self._response, 'http_verb', '')
            self.req_args = getattr(self._response, 'req_args', '')
            self.status_code = getattr(self._response, 'status_code', '')

    def error(self):
        if self._response:
            return self.__error

        if self._reason:
            return self.strerror

        self._log.warn(f'{NotImplemented}')
        return f'{self._error}'

    def needed(self):
        if self._response:
            return self._needed

        if self._reason:
            return self.strerror

        self._log.warn(f'{NotImplemented}')
        return f'{self._error}'

    def __repr__(self):
        if self._response:
            return f'{self.data}'

        if self._reason:
            return f'{self.strerror}'

        self._log.warn(f'{NotImplemented}')
        return f'{self._error}'

    def __str__(self):
        return self.__repr__()



import os
import warnings
import selenium

from automon.log import Logging
from automon.helpers.osWrapper.environ import environ

log = Logging(name='SeleniumConfig', level=Logging.INFO)


class SeleniumConfig(object):

    def __init__(self, webdriver=None, chromedriver: str = None):
        self.webdriver = webdriver or selenium.webdriver
        self.selenium_chromedriver_path = chromedriver or environ('SELENIUM_CHROMEDRIVER_PATH', '')

        if self.selenium_chromedriver_path:
            os.environ['PATH'] = f"{os.getenv('PATH')}:{self.selenium_chromedriver_path}"

    def __repr__(self):
        return f'{self.__dict__}'

    def chrome(self):
        """Chrome with no options

        """
        opt = SeleniumOptions(self.webdriver)
        opt.default()

    def chrome_maximized(self):
        """Chrome with no options

        """
        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.maximized()

    def chrome_fullscreen(self):
        """Chrome with no options

        """
        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.fullscreen()

    def chrome_for_docker(self):
        """Chrome best used with docker

        """
        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.nosandbox()
        opt.headless()
        opt.noinfobars()
        opt.noextensions()
        opt.nonotifications()

    def chrome_sandboxed(self):
        """Chrome with sandbox enabled

        """
        warnings.warn('Docker does not support sandbox option')
        warnings.warn('Default shm size is 64m, which will cause chrome driver to crash.', Warning)

        opt = SeleniumOptions(self.webdriver)
        opt.default()

    def chrome_nosandbox(self):
        """Chrome with sandbox disabled

        """
        warnings.warn('Default shm size is 64m, which will cause chrome driver to crash.', Warning)

        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.nosandbox()

    def chrome_headless_sandboxed(self):
        """Headless Chrome with sandbox enabled

        """
        warnings.warn('Docker does not support sandbox option')
        warnings.warn('Default shm size is 64m, which will cause chrome driver to crash.', Warning)

        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.headless()

    def chrome_headless_nosandbox(self):
        """Headless Chrome with sandbox disabled

        """
        warnings.warn('Default shm size is 64m, which will cause chrome driver to crash.', Warning)

        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.headless()
        opt.nosandbox()

    def chrome_headless_nosandbox_unsafe(self):
        """Headless Chrome with sandbox disabled with not certificate verification

        """
        warnings.warn('Default shm size is 64m, which will cause chrome driver to crash.', Warning)

        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.headless()
        opt.nosandbox()
        opt.unsafe()

    def chrome_headless_nosandbox_noshm(self):
        """Headless Chrome with sandbox disabled

        """
        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.headless()
        opt.nosandbox()
        opt.noshm()

    def chrome_headless_nosandbox_bigshm(self):
        """Headless Chrome with sandbox disabled

        """
        warnings.warn('Larger shm option is not implemented', Warning)

        opt = SeleniumOptions(self.webdriver)
        opt.default()
        opt.headless()
        opt.nosandbox()
        opt.bigshm()

    def chrome_remote(self, host: str = '127.0.0.1', port: str = '4444', executor_path: str = '/wd/hub'):
        """Remote Selenium

        """
        log.info(
            f'Remote WebDriver Hub URL: http://{host}:{port}{executor_path}/static/resource/hub.html')

        self.webdriver.Remote(
            command_executor=f'http://{host}:{port}{executor_path}',
            desired_capabilities=selenium.webdriver.common.desired_capabilities.DesiredCapabilities.CHROME
        )


class SeleniumOptions:

    def __init__(self, webdriver):
        self.webdriver = webdriver or selenium.webdriver
        self.options = self.webdriver.ChromeOptions()

    def default(self):
        self.options.add_argument('start-maximized')

    def unsafe(self):
        warnings.warn('Certificates are not verified', Warning)
        self.options.add_argument('--ignore-certificate-errors')

    def nosandbox(self):
        self.options.add_argument('--no-sandbox')

    def headless(self):
        self.options.add_argument('headless')

    def noshm(self):
        warnings.warn('Disabled shm will use disk I/O, and will be slow', Warning)
        self.options.add_argument('--disable-dev-shm-usage')

    def bigshm(self):
        warnings.warn('Big shm not yet implemented', Warning)

    def noinfobars(self):
        self.options.add_argument("--disable-infobars")

    def noextensions(self):
        self.options.add_argument("--disable-extensions")

    def maximized(self):
        self.options.add_argument("--start-maximized")

    def fullscreen(self):
        self.options.add_argument("--start-fullscreen")

    def nonotifications(self):
        # Pass the argument 1 to allow and 2 to block
        self.options.add_experimental_option(
            "prefs", {"profile.default_content_setting_values.notifications": 1}
        )



import json
import datetime

from dateutil import parser

from automon.log import Logging

log = Logging('AbstractDataType', level=Logging.DEBUG)


class AbstractDataType:
    def __init__(self, data: dict):
        self.__dict__.update(data)

    def __repr__(self):
        return self.to_json()

    def __len__(self):
        return self.id

    def __lt__(self, other):
        return self._created() < other._created()

    def _created(self):
        return self._parse_timestamp(self.create_time)

    def _parse_timestamp(self, timestamp: str) -> datetime.datetime:
        """parse string into datetime object

        :timestamp: '2020-02-18T15:50:58.903568Z'
        :returns: datetime.datetime(2020, 2, 18, 15, 50, 58, 903568, tzinfo=tzutc())
        """
        return parser.parse(timestamp)

    def to_dict(self):
        return {k: v for k, v in self.__dict__.items() if v}

    def to_json(self):
        return json.dumps(self.to_dict())



import queue
import threading
import time

print_lock = threading.Lock()


def init_queue():
    """ Initialize the Queue """
    return queue.Queue()


def job(queue_item):
    """ What is actually being worked on """

    with print_lock:
        print('+ Job:', threading.current_thread().getName())

    args = dict()
    for key in queue_item:
        if key == 'headers':
            args['headers'] = queue_item[key]
            # Disabled Neo4j
            # http.http_header(**args)

    else:
        args = queue_item


def worker(queue_list):
    """ Worker puts the things to work """

    while True:

        queue_item = queue_list.get()

        if queue_item:
            with print_lock:
                print('+ Worker:', threading.current_thread().getName())

            # TODO: Find a way to figure out what data is before doing work on it
            # TODO: Add failure routine if queue_item is a str and not a list

            job(queue_item)

            queue_list.task_done()

        else:
            return


def start_worker(queue_list):
    """ This starts all the threading """

    print('+ Queue:', queue_list.qsize())

    try:
        num_worker_threads = 100 if queue_list.qsize() > 100 else queue_list.qsize()
        threads = []
        for _ in range(num_worker_threads):
            t = threading.Thread(target=worker, args=(queue_list,))
            t.setDaemon(True)
            t.start()
            threads.append(t)

        start = time.time()

        print('+ Threads started:', len(threads))
        for _ in threads:
            print('`-', _)

        # queue_list.join()  # block until all tasks are done

        print('+ Entire job took:', time.time() - start)

        print('+ Threads enumerate:', len(threading.enumerate()))
        for _ in threading.enumerate():
            print('`-', _)

        for _ in range(num_worker_threads):  # stop workers
            queue_list.put(None)
        # for _ in threading:
        #     _.join()

        # print(threading.current_thread().getName(), 'is', threading.Thread.is_alive(threading.current_thread()))

    except:
        raise



import unittest

from automon.helpers.networking import *


class NetworkingTest(unittest.TestCase):
    def test_Networking(self):
        self.assertTrue(Networking)

    def test_check_connection(self):
        self.assertFalse(Networking.check_connection('x://localhost:0'))
        self.assertFalse(Networking.check_connection('localhost:0'))
        self.assertFalse(Networking.check_connection('localhost:0', timeout=0))

        if Networking.check_connection('x://1.1.1.1:443'):
            self.assertTrue(Networking.check_connection('https://www.google.com:443'))
            self.assertTrue(Networking.check_connection('x://www.google.com:443'))

    def test_urlparse(self):
        self.assertTrue(Networking.urlparse('x://localhost:0'))


if __name__ == '__main__':
    unittest.main()



import subprocess

from pprint import pprint
from subprocess import PIPE

from automon.log.logger import Logging
from automon.helpers.dates import Dates

log = Logging(name='Run', level=Logging.DEBUG)


class Run:

    def __init__(self, command: str = None, *args, **kwargs):
        """Run shell"""

        self.last_run = None
        self.command = ''

        self.stdout = b''
        self.stderr = b''

        self.call = None
        self.returncode = None

        if command:
            self.run(command=command, *args, **kwargs)

    def rc(self):
        if self.call:
            self.returncode = self.call.returncode
            return self.returncode

    def Popen(self, *args, **kwargs):
        return self.run(*args, **kwargs)

    def pretty(self):
        return pprint(self.stdout.decode())

    def print(self):
        return print(self.stdout.decode())

    def set_command(self, command: str) -> bool:
        if command:
            self.command = command
            return True
        return False

    def which(self, program: str, *args, **kwargs) -> bool:
        """runs which

        :param program:
        :return:
        """
        if program:
            return self.run(command=f'which {program}', *args, **kwargs)
        return False

    def run_command(self, *args, **kwargs) -> bool:
        """alias to run"""
        return self.run(*args, **kwargs)

    def run(self, command: str = None,
            text: bool = False,
            inBackground: bool = False,
            shell: bool = False,
            **kwargs) -> bool:

        if command:
            command = self._command(command)

        elif self.command:
            command = self._command(self.command)

        try:
            if inBackground:
                self.call = subprocess.Popen(command, text=text, shell=shell, **kwargs)
                return True
            else:
                self.call = subprocess.Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE, text=text, shell=shell,
                                             **kwargs)
                stdout, stderr = self.call.communicate()
                # call.wait()

                timestamp = Dates.iso()

                self.last_run = timestamp
                self.stdout = stdout
                self.stderr = stderr
                self.returncode = self.call.returncode

                if self.stdout:
                    log.debug(f'[stdout] {stdout}')

                if self.stderr:
                    log.error(f'[stderr] {stderr}', enable_traceback=False)

                if self.returncode == 0:
                    return True

        except Exception as e:
            self.stderr = f'{e}'.encode()
            log.error(f'{e}', enable_traceback=False)

        return False

    def _command(self, command: str) -> list:
        log.debug(f'[_command] {command}')
        if isinstance(command, str):
            split_command = f'{command}'.split(' ')
        self.command = split_command

        for arg in split_command:
            if '|' in arg:
                log.warn(f'Pipes are not supported! {split_command}')

        return self.command

    def __repr__(self) -> str:
        return f'{self.command} stderr: ({len(self.stderr) / 1024} Kb) stdout: ({round(len(self.stdout) / 1024, 2)} Kb)'

    def __len__(self):
        return sum([len(self.stdout), len(self.stderr)])

    def __eq__(self, other):
        if isinstance(other, Run):
            self.command == other.command
            return True
        return False



import unittest

from automon.helpers.assertions import make_tuple, assert_tuple, assert_string, assert_dict, assert_list


class AssertionsTest(unittest.TestCase):
    def test_make_tuple(self):
        self.assertTupleEqual(make_tuple('"test",'), ('test',))
        self.assertTupleEqual(make_tuple('"test","test"'), ('test', 'test'))
        self.assertRaises(ValueError, make_tuple, 'test')
        self.assertRaises(ValueError, make_tuple, 'test,')
        self.assertRaises(ValueError, make_tuple, 'test,test')

    def test_assert_tuple(self):
        self.assertFalse(assert_tuple('test'))
        self.assertFalse(assert_tuple(1))
        self.assertFalse(assert_tuple(None))
        self.assertFalse(assert_tuple(object))
        self.assertFalse(assert_tuple(''))
        self.assertFalse(assert_tuple({}))
        self.assertFalse(assert_tuple(('test')))
        self.assertFalse(assert_tuple({1: 2}))
        self.assertTrue(assert_tuple(()))
        self.assertTrue(assert_tuple(('test',)))

    def test_assert_dict(self):
        self.assertFalse(assert_dict('test'))
        self.assertFalse(assert_dict({'test'}))
        self.assertTrue(assert_dict({'test': 'test'}))

    def test_assert_list(self):
        self.assertFalse(assert_list('test'))
        self.assertTrue(assert_list(['test']))
        self.assertTrue(assert_list([]))

    def test_assert_string(self):
        self.assertFalse(assert_string([]))
        self.assertFalse(assert_string(1))
        self.assertFalse(assert_string({}))
        self.assertFalse(assert_string(dict()))
        self.assertFalse(assert_string(tuple()))
        self.assertRaises(TypeError, assert_string)
        self.assertTrue(assert_string('test'))


if __name__ == '__main__':
    unittest.main()



from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains

from automon.log.logger import Logging
from automon.helpers.sleeper import Sleeper
from automon.integrations.seleniumWrapper.config import SeleniumConfig
from automon.integrations.seleniumWrapper.browser import SeleniumBrowser

from automon.integrations.minioWrapper import MinioClient

log = Logging(name='instagram', level=Logging.DEBUG)


def authenticate(username, password, minio_client=None, retries=None):
    """Authenticates through browser and returns browser driver

    :param username: username string
    :param password: password string
    :param retries: not implemented
    :return: authenticated browser
    """

    while True:

        # TODO: create capture proxy
        #       send traffic to /api
        login_page = 'https://www.instagram.com/accounts/login/?source=auth_switcher'

        # browser = SeleniumBrowser(chrome())
        # browser = SeleniumBrowser(chrome_headless_nosandbox())
        browser = SeleniumBrowser(SeleniumConfig.chrome_for_docker())
        # browser = SeleniumBrowser(chrome_sandboxed())
        # browser = SeleniumBrowser(chrome_headless_sandboxed())
        # browser = SeleniumBrowser(chrome_remote())

        if minio_client:
            browser.set_minio_client(minio_client)

        browser.browser.get(login_page)

        log.logging.debug('[authenticate] {}'.format(login_page))

        Sleeper.seconds('instagram get page', 1)

        actions = ActionChains(browser.browser)
        actions.send_keys(Keys.TAB)
        actions.send_keys(username)
        actions.perform()

        Sleeper.seconds('instagram get page', 1)

        # the password field is sometimes div[3] and div[4]
        login_pass_xpaths = [
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[3]/div/label/input',
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[4]/div/label/input'
        ]

        login_btn_xpaths = [
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[4]/button',
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[6]/button'
        ]

        found_pass = False
        for xpath in login_pass_xpaths:
            try:
                login_pass = browser.browser.find_element_by_xpath(xpath)
                found_pass = True
                break
            except:
                pass

        Sleeper.seconds('instagram get page', 2)

        found_btn = False
        for xpath in login_btn_xpaths:
            try:
                login_btn = browser.browser.find_element_by_xpath(xpath)
                found_btn = True
                break
            except:
                pass

        if found_pass and found_btn:
            break
        else:
            log.logging.error('[browser] Authentication failed')

            log.logging.debug(
                '[browser] Found password field: {} Found login button: {}'.format(browser.browser.name, found_pass,
                                                                                   found_btn))

            Sleeper.minute("instagram can't authenticate")

    login_pass.send_keys(password)
    login_btn.click()

    Sleeper.seconds('wait for instagram to log in', 5)

    log.logging.debug(
        '[authenticated browser] [{}] {} session: {}'.format(browser.browser.name, browser.browser.title,
                                                             browser.browser.session_id))
    browser.save_screenshot_to_minio()

    return browser


def get_stories(authenticated_browser, account):
    """ Retrieve story
    """
    story = 'https://www.instagram.com/stories/{}/'.format(account)
    num_of_stories = 0
    # TODO: set browser to redirect to proxy here
    # TODO: check if account exists
    browser = authenticated_browser

    log.logging.debug('[get_stories] {}'.format(story))

    browser.browser.get(story)
    browser.save_screenshot_to_minio(prefix=account)

    if 'Page Not Found' in browser.browser.title:
        log.logging.debug('[get_stories] no stories for {}'.format(account))
        return num_of_stories

    Sleeper.seconds('instagram', 2)

    while True:
        try:
            next_story(browser)

            title = browser.browser.title
            if title == 'Instagram':
                log.logging.debug(('[get_stories] {} end of stories'.format(account)))
                raise Exception
            num_of_stories += 1
            Sleeper.seconds('watch the story for a bit', 1)
            browser.save_screenshot_to_minio(prefix=account)
        except:
            # TODO: disable browser proxy when done
            log.logging.debug('[get_stories] done: {}'.format(account))
            return num_of_stories


def next_story(authenticated_browser):
    """ Click next story button
    """

    xpaths = [
        '//*[@id="react-root"]/section/div/div/section/div[2]/div[1]/div/div/div[2]/div/div/button',
        '//*[@id="react-root"]/section/div/div/section/div[2]/button[2]'
    ]

    found_btn = False
    for xpath in xpaths:
        try:
            browser = authenticated_browser
            button = browser.browser.find_element_by_xpath(xpath)
            found_btn = True
            log.logging.debug('[next_story] next story')
            return button.click()
        except:
            pass

    if not found_btn:
        # no more stories. exit
        log.logging.debug('[next_story] no more stories')
        raise Exception


def get_page(authenticated_browser, account):
    """ Get page
    """
    # TODO: need to download page
    log.logging.debug('[get_page] getting {}'.format(account))
    page = 'https://instagram.com/{}'.format(account)
    browser = authenticated_browser
    return browser.browser.get(page)


def run(config):
    client = minio.client(config['minio-hev'], secure=False)

    instagram_config = config['instagram']
    login = instagram_config['login']['account']
    password = instagram_config['login']['password']
    accounts = instagram_config['following']

    log.logging.debug('[login] {}'.format(login))
    log.logging.info('Running...')
    log.logging.info('[accounts] {}'.format(len(accounts)))

    while True:

        if len(accounts) > 0:

            browser = authenticate(login, password, client)

            for account in accounts:

                while True:
                    if runrun(browser, account):
                        break
                    else:
                        browser = authenticate(login, password, client)

        Sleeper.hour('instagram')


def runrun(browser, account):
    log.logging.debug(
        '[runrun] [{}] {} session: {}'.format(browser.browser.name, browser.browser.title,
                                              browser.browser.session_id))

    num_of_stories = get_stories(browser, account)

    log.logging.info('[{}] {} stories'.format(account, num_of_stories))

    # Sleeper.minute('instagram')

    return True


def test_run(config):
    client = minio.client(config['minio-hev'], secure=False)

    instagram_config = config['instagram']
    login = instagram_config['login']['account']
    password = instagram_config['login']['password']
    accounts = instagram_config['following']

    log.logging.debug('[login] {}'.format(login))
    log.logging.info('Running...')
    log.logging.info('[accounts] {}'.format(len(accounts)))

    while True:

        if len(accounts) > 0:

            browser = authenticate(login, password, client)

            for account in accounts:

                while True:
                    if runrun(browser, account):
                        break
                    else:
                        browser = authenticate(login, password, client)

                    break
                break
            break



import os
import re
import time
import warnings
import datetime

from swiftclient.service import SwiftService

from automon.log import Logging
from automon.integrations.swift.error import SwiftError_
# from automon.integrations.swift.config import SwiftConfig
from automon.integrations.swift.iterables import SwiftList, SwiftItem

log = Logging(__name__, Logging.INFO)

Logging('requests', Logging.CRITICAL)
Logging('swiftclient', Logging.CRITICAL)


class SwiftClient:
    def __init__(self):
        self._log = Logging(SwiftClient.__name__, Logging.DEBUG)

    @staticmethod
    def list_summary(files, folders):
        return files, folders

    def list_container(self, container, summary=False, filter='',
                       separate=False) -> [SwiftItem]:

        self._log.info(f'listing {container} (filter: {filter})')

        swift_objects = []
        swift_objects_v2 = []
        files = []
        files_v2 = []
        folders = []
        folders_v2 = []

        for page in SwiftList(container).list_gen():

            for item in page.list_gen():
                item_v1 = item.data()
                item_v2 = item

                if filter:
                    if not item.filter(filter):
                        continue

                swift_objects.append(item_v1)
                swift_objects_v2.append(item_v2)

                if item.is_directory():
                    folders.append(item_v1)
                    folders_v2.append(item_v2)
                else:
                    files.append(item_v1)
                    files_v2.append(item_v2)

        self._log.debug(f'Listing for {container}: {len(swift_objects)} objects')

        if summary:
            return len(swift_objects)

        if separate:
            self.list_summary(files, folders)

        return swift_objects

    def cleanup(self, container, days=7):

        msg = (f'Starting cleanup \n'
               f'>Retention policy: {days} days \n')
        self._log.info(msg)

        today = datetime.date.today()
        start_time = time.time()

        # Delete backups older than 7 days
        cleanup = self.list_container(container)
        total = len(cleanup)
        pending_deletion = []
        dates_to_retrain = []

        # keep one week of data
        for day in range(days + 1):
            # '2020-04-01'
            keep = today - datetime.timedelta(days=day)
            dates_to_retrain.append(keep)

        progress = 0

        # create a list of items to delete
        for item in cleanup:

            progress += 1
            percent = format(progress / total * 100, '.2f')

            item_s = SwiftItem(item)
            name = item_s.name

            found = False
            # check if item name is within date range
            for date in dates_to_retrain:
                # '^2020-04-01'
                regex = f'^{date}'

                if re.search(regex, item_s.name):
                    self._log.info(f"{percent}% ({progress}/{total}) cleanup retain: \'{regex}\', {name}")
                    found = True
                    break

            if not found:
                pending_deletion.append(item)
                self._log.info(f"{percent}% ({progress}/{total}) pending deletion: {name}")

        elapsed_time = time.time() - start_time
        minutes = int(elapsed_time / 60)

        msg = (f'>Created deletion list for past {days} days (Deleting: {len(pending_deletion)} objects)\n'
               f">Cleanup has been running for: {minutes} minutes")
        self._log.debug(msg)

        objects = 0
        folders = 0
        total = len(pending_deletion)
        progress = 0

        # create stats on what will be deleted
        for item in pending_deletion:
            item_s = SwiftItem(item)
            name = item_s.name

            progress += 1
            percent = format(progress / total * 100, '.2f')
            elapsed_time = time.time() - start_time
            minutes = int(elapsed_time / 60)

            if item_s.is_directory():
                folders += 1
            else:
                objects += 1

            self._log.info(f'{percent}% ({progress}/{total}) summarizing deletion: {name}')

        msg = (f'Cleaning up older than {days} days: \n'
               f'>{total} *total objects* \n'
               f'>{objects} *objects* \n'
               f'>{folders} *folders* \n'
               f'see debug messages')
        self._log.debug(msg)

        progress = 0

        # delete objects
        for item in pending_deletion:
            name = SwiftItem(item).name

            progress += 1
            percent = format(progress / total * 100, '.2f')
            elapsed_time = time.time() - start_time
            minutes = int(elapsed_time / 60)

            # this does the actual deletion
            self.delete_object(container, item)
            self._log.info(f'{percent}% ({progress}/{total}) deleted: {name}')

            if progress % 10000 == 0 or progress % total == 0:
                elapsed_time = time.time() - start_time
                minutes = int(elapsed_time / 60)

                msg = (f'>Deletion is currently at `{percent}%` ({progress}/{total})\n'
                       f">Backup has been running for: {minutes} minutes")
                self._log.debug(msg)

        if pending_deletion:
            msg = ('Cleanup Finished\n'
                   f">It took: {minutes} minutes")
            self._log.debug(msg)

    def backup(self, source, destination, test=None, skip_known=True):

        # TODO: for some reason it's finished time shows 3 minutes, when debug shows 90 minutes

        today = f'{datetime.date.today()}'
        start_time = time.time()
        progress = 0
        retries = 0

        msg = f'Backup {source} started'
        self._log.debug(msg)

        items = self.list_container(source)
        backups = self.list_container(destination, filter=today)
        total = len(items)

        source_stats = self.stats(source, show_types=True)

        for item in items:

            progress += 1

            s_item = SwiftItem(item)
            name = s_item.name

            if test:
                regex = str(test)
                if re.search(regex, item):
                    self._log.debug(f'Test match: {test} \n>{item}')

                else:
                    continue

            percent = format(progress / total * 100, '.2f')

            # skip items that are already backed up
            if skip_known:
                exists = False
                for b in backups:
                    b = SwiftItem(b)
                    if f'{today}/{s_item.name}' == b.name:
                        self._log.info(f'{percent}% ({progress}/{total}) exists, skipping {s_item.name}')
                        exists = True
                        break
                if exists:
                    continue

            if progress % 10000 == 0 or progress % total == 0:
                elapsed_time = time.time() - start_time
                minutes = int(elapsed_time / 60)
                msg = (f'>Backup {source} is currently at `{percent}%` ({progress}/{total})\n'
                       f">Backup has been running for: {minutes} minutes")
                self._log.debug(msg)

                self.stats(destination, filter=today)

            retry = True
            while True:
                with SwiftService() as swift:
                    try:
                        if s_item.is_directory():

                            folder = f'{today}/{name}'
                            os.makedirs(folder, exist_ok=True)
                            compatible_name = os.path.split(name)[0]

                            options = {"destination": f"/{destination}/{today}/{compatible_name}"}

                            try:
                                for i in swift.upload(destination, [folder], options):

                                    if i["success"]:
                                        self._log.info(
                                            f'{percent}% ({progress}/{total}) '
                                            f'created directory /{destination}/{today}/{name}')

                                        retry = False

                                    if "error" in i and isinstance(i["error"], Exception):
                                        self._log.error(f'{SwiftError_(i)}')

                                        retries += 1

                            except Exception as _:
                                self._log.error(item)

                                retries += 1

                            # TODO: find a thread-safe method to delete folders
                            # shutil.rmtree(str(today))

                        else:

                            options = {"destination": f"/{destination}/{today}/{name}"}

                            for i in swift.copy(source, [name], options):

                                if test:
                                    retry = False

                                if i["success"]:
                                    if i["action"] == "copy_object":
                                        self._log.info(
                                            f'{percent}% ({progress}/{total}) '
                                            f'copied {i["destination"]} from /{i["container"]}/{i["object"]}')

                                    if i["action"] == "create_container":
                                        self._log.info(
                                            f'{percent}% ({progress}/{total}) '
                                            f'container {i["container"]} created')

                                    retry = False

                                else:
                                    if "error" in i and isinstance(i["error"], Exception):

                                        if 'Authorization Failure. Authorization failed' in str(i):
                                            error = (
                                                f'''This error happens every once in a while. I'm not really sure why, 
                                                but might be some kind of stale timeout when SwiftService() isn't doing 
                                                anything \n'''
                                                f'{SwiftError_(i)}'
                                            )
                                        else:
                                            error = f'{SwiftError_(i)}'

                                        log.error(error)
                                        retries += 1

                    except Exception as _:
                        log.error(item)
                        retries += 1

                if not retry:
                    break

        self._log.info('building backup summary')

        source_total_objects, \
        source_total_dirs, \
        source_objects, _, _ = source_stats

        filter_destination_total_objects, \
        filter_destination_total_dirs, \
        filter_objects, _, _ = self.stats(destination, post_log=True, filter=today)

        destination_total_objects, \
        destination_total_dirs, \
        destination_objects, _, _ = self.stats(destination, post_log=False)

        # elapsed_time = time.time() - start_time
        # minutes = int(elapsed_time / 60)

        msg = (
            f'*Backup Finished* \n'
            f'Stats for {source}: \n'
            f'>{source_total_objects} *total objects* \n'
            f'>{source_objects} *objects* \n'
            f'>{source_total_dirs} *dirs* \n'
            f'Stats for {destination} ({today}): \n'
            f'>{filter_destination_total_objects} *total objects* \n'
            f'>{filter_objects} *objects* \n'
            f'>{filter_destination_total_dirs} *dirs* \n'
            f'Stats for all {destination}: \n'
            f'>{destination_total_objects} *total objects* \n'
            f'>{destination_objects} *objects* \n'
            f'>{destination_total_dirs} *dirs* \n')
        log.debug(msg)

        # TODO: new files may be added during the backup, so need verify only the initial files being backed up

        # missing = self.find_missing(source, destination, filter=today, post_log=False)
        # missing_count = len(missing)
        # if missing:
        #     try:
        #         self.log.info(
        #             f'Missing {missing_count} objects'
        #             '>'.join(missing)
        #         )
        #     except Exception as e:
        #         # debug testing
        #         slacklog.error(f'{e}')
        # else:
        #     slacklog.info(msg)
        #
        #     msg = ('*Backup Finished* \n'
        #            f'>It took: {minutes} minutes ({retries} retries)\n')
        #     self.log.debug(msg)

    def find_missing(self, source, destination, filter, post_log=True):

        msg = f'Verifying backup {destination} {filter} \n'

        missing_objects = ()
        missing_objects_list = []

        today = datetime.date.today()

        src_list = self.list_container(source)
        dst_list = self.list_container(destination, filter=filter)

        total = len(src_list)

        progress = 0

        for a_item in src_list:

            a_name = SwiftItem(a_item).name

            progress += 1
            percent = format(progress / total * 100, '.2f')

            found = False
            for b_item in dst_list:
                b_name = SwiftItem(b_item).name
                if f'{today}/{a_name}' == b_name:
                    found = True
                    break
            if not found:
                self._log.info(f'{percent}% ({progress}/{total}) backup missing: {a_name}')
                missing_objects + (tuple(f' * {a_item} \n'))
                missing_objects_list.append(a_item)
            else:
                self._log.info(f'{percent}% ({progress}/{total}) verified, {a_name}')

        log.debug(f'missing_objects: {len(missing_objects)}')

        if missing_objects:
            msg = (f'Missing {len(missing_objects)} objects: \n'
                   '>List of missing: \n'
                   ''.join(missing_objects))

        return missing_objects_list

    def stats(self, container, filter='', post_log=True, show_types=False):

        self._log.info(f'stat {container} (filter: {filter})')

        list_items = self.list_container(container, filter=filter)
        total_dirs = 0
        list_types = {}

        for item in list_items:

            s_item = SwiftItem(item)

            key = s_item.content_type
            if key in list_types.keys():
                list_types[key] += 1
            else:
                list_types[key] = 1

            if s_item.is_directory():
                total_dirs += 1

        list_types = {k: v for k, v in sorted(list_types.items(), key=lambda item: item[1], reverse=True)}

        total_objects = len(list_items)
        objects = total_objects - total_dirs

        msg = ((f'Stats for {container}: \n'
                f'>{total_objects}*total objects* \n'
                f'>{objects}*objects* \n'
                f'>{total_dirs}*dirs* \n'))

        if filter:
            msg = ((f'Stats for {container} ({filter}): \n'
                    f'>{total_objects}*total objects* \n'
                    f'>{objects}*objects* \n'
                    f'>{total_dirs}*dirs* \n'))

        if show_types:
            msg = msg + f'>*types:* `{list_types}` \n'

        return total_objects, total_dirs, objects, list_items, list_types

    def delete_object(self, container, item):

        name = SwiftItem(item).name

        with SwiftService() as swift:
            try:
                for i in swift.delete(container=container, objects=[name]):
                    if i['success']:
                        self._log.info(f'deleted: {name}')

            except Exception as e:
                log.error(f'{e}')

    def delete(self, container, filter):

        deletion = self.list_container(container, filter=filter)

        deletion_count = len(deletion)
        progress = 0

        for item in deletion:

            name = SwiftItem(item).name

            progress += 1
            percent = format(progress / deletion_count * 100, '.2f')

            with SwiftService() as swift:
                try:
                    for i in swift.delete(container=container, objects=[name]):
                        if i['success']:
                            self._log.info(f'{percent}% ({progress}/{deletion_count}) deleted: {name}')

                except Exception as e:
                    log.error(f'{e}')

        log.debug(
            f'Deletion summary: \n>container: {container} \n>filter: {filter} \n>{deletion_count} objects deleted')

    def delete_container(self, container):

        with SwiftService() as swift:
            try:

                for _ in swift.delete(container):
                    self._log.info(f'deleting container: {container}')

            except Exception as e:
                log.error(f'{e}')

    def restore(self):
        return warnings.warn(NotImplemented)



import sys
import unittest

from automon.integrations.mac.airport import Airport

from automon.integrations.neo4jWrapper import Neo4jClient


class AirportToNeo4jTest(unittest.TestCase):
    a = Airport()
    n = Neo4jClient()

    def test_scan_xml(self):
        if self.a.isReady():
            test = self.a.scan_xml()
            if test:
                self.assertTrue(test)

        if self.n.isConnected():
            # self.n.delete_all()
            for bssid in self.a.ssids:
                flatten = bssid._ssid
                flatten.update(bssid.__dict__)
                flatten.pop('_ssid')

                self.assertTrue(self.n.merge_dict(
                    prop='BSSID',
                    value=bssid.BSSID,
                    data=flatten,
                    label='BSSID',
                ))

                self.assertTrue(self.n.relationship(
                    A_prop='SSID', A_value=bssid.SSID,
                    B_prop='SSID', B_value=bssid.SSID,
                    WHERE=f'A.BSSID <> B.BSSID', label='EXTENDS'
                ))


if __name__ == '__main__':
    unittest.main()



import json
import base64
import functools

from typing import Optional

from automon.log import Logging
from automon.integrations.requests import Requests

from .artifact import Artifact
from .config import SplunkSoarConfig
from .container import Container
from .vault import Vault
from .rest import Urls
from .responses import (
    AppRunResults,
    AppRunResponse,
    CancelPlaybookResponse,
    CloseContainerResponse,
    CreateContainerAttachmentResponse,
    CreateContainerResponse,
    GenericResponse,
    PlaybookRun,
    Response,
    RunPlaybookResponse,
    UpdatePlaybookResponse,
    VaultResponse
)

log = Logging(name='SplunkSoarClient', level=Logging.DEBUG)
Logging(name='RequestsClient', level=Logging.DEBUG)


class SplunkSoarClient:
    def __init__(self, host: str = None,
                 user: str = None,
                 password: str = None,
                 config: SplunkSoarConfig = None):
        """Splunk SOAR Client"""

        self.config = config or SplunkSoarConfig(host=host, user=user, password=password)
        self.client = Requests(headers=self.config.headers)

        self.action_run = None
        self.app = None
        self.app_run = None
        self.asset = None
        self.containers = None
        self.playbook_run = None

    def __repr__(self) -> str:
        return f'{self.__dict__}'

    def _content(self) -> bytes:
        """get result"""
        if self.client.results:
            return self.client.results.content
        return b''

    def _content_dict(self) -> dict:
        """convert request.content to dict"""
        if self._content():
            return json.loads(self._content())
        return {}

    def _get(self, url: str) -> bool:
        """send get request"""
        return self.client.get(url=url, headers=self.client.headers)

    def _isConnected(func):
        """wrapper for connection checking"""

        @functools.wraps(func)
        def wrapper(self, *args, **kwargs):
            if self.config.isReady():
                if self._get(Urls.container(page_size=1)):
                    return func(self, *args, **kwargs)
            return False

        return wrapper

    def _delete(self, url: str) -> bool:
        """send get request"""
        return self.client.delete(url=url, headers=self.client.headers)

    def _post(self, url: str, data: dict) -> bool:
        """send post request"""
        return self.client.post(url=url, headers=self.client.headers, data=data)

    @_isConnected
    def close_container(self, container_id: int, **kwargs) -> Optional[CloseContainerResponse]:
        """Set container status to closed"""
        data = dict(status='closed')
        if self._post(Urls.container(identifier=container_id, **kwargs), data=json.dumps(data)):
            if self.client.results.status_code == 200:
                response = CloseContainerResponse(self._content_dict())
                log.info(f'container closed: {response}')
                return response

        log.error(msg=f'close failed. {self.client.to_dict()}', raise_exception=False)

    @_isConnected
    def cancel_playbook_run(
            self,
            playbook_run_id: int = None,
            cancel: bool = True, **kwargs) -> CancelPlaybookResponse:
        """Cancel playbook run"""
        data = dict(cancel=cancel)
        data = json.dumps(data)

        if self._post(Urls.playbook_run(identifier=playbook_run_id, **kwargs), data=data):
            if self.client.results.status_code == 200:
                response = CancelPlaybookResponse(self._content_dict())
                log.info(f'cancel playbook run: {response}')
                return response

        log.error(f'cancel failed: {playbook_run_id} {self.client.to_dict()}', enable_traceback=False)

    @_isConnected
    def create_artifact(
            self,
            container_id,
            cef=None,
            cef_types=None,
            data=None,
            description=None,
            end_time=None,
            ingest_app_id=None,
            kill_chain=None,
            label=None,
            name=None,
            owner_id=None,
            run_automation=None,
            severity=None,
            source_data_identifier=None,
            start_time=None,
            tags=None,
            type=None,
            *args, **kwargs) -> Artifact:
        """Create artifact"""

        artifact = Artifact(
            dict(cef=cef,
                 cef_types=cef_types,
                 container_id=container_id,
                 data=data,
                 description=description,
                 end_time=end_time,
                 ingest_app_id=ingest_app_id,
                 kill_chain=kill_chain,
                 label=label,
                 name=name,
                 owner_id=owner_id,
                 run_automation=run_automation,
                 severity=severity,
                 source_data_identifier=source_data_identifier,
                 start_time=start_time,
                 tags=tags,
                 type=type)
        )

        if self._post(Urls.artifact(*args, **kwargs), data=artifact.to_json()):
            if self.client.results.status_code == 200:
                id = self.client.to_dict()['id']
                log.info(f'artifact created. {artifact} {self.client.to_dict()}')
                return self.get_artifact(artifact_id=id)
            else:
                existing_artifact_id = self.client.to_dict()['existing_artifact_id']
                log.info(f'artifact exists. {artifact} {self.client.to_dict()}')
                return self.get_artifact(artifact_id=existing_artifact_id)

        log.error(f'create artifact. {self.client.to_dict()}', enable_traceback=False)
        return False

    @_isConnected
    def create_container(
            self,
            label,
            name,
            artifacts=None,
            asset_id=None,
            close_time=None,
            custom_fields=None,
            data=None,
            description=None,
            due_time=None,
            end_time=None,
            ingest_app_id=None,
            kill_chain=None,
            owner_id=None,
            role_id=None,
            run_automation=None,
            sensitivity=None,
            severity=None,
            source_data_identifier=None,
            start_time=None,
            open_time=None,
            status=None,
            tags=None,
            tenant_id=None,
            container_type=None,
            template_id=None,
            authorized_users=None,
            *args, **kwargs) -> Container:
        """Create container"""

        container = Container(
            dict(label=label,
                 name=name,
                 artifacts=artifacts,
                 asset_id=asset_id,
                 close_time=close_time,
                 custom_fields=custom_fields,
                 data=data,
                 description=description,
                 due_time=due_time,
                 end_time=end_time,
                 ingest_app_id=ingest_app_id,
                 kill_chain=kill_chain,
                 owner_id=owner_id,
                 role_id=role_id,
                 run_automation=run_automation,
                 sensitivity=sensitivity,
                 severity=severity,
                 source_data_identifier=source_data_identifier,
                 start_time=start_time,
                 open_time=open_time,
                 status=status,
                 tags=tags,
                 tenant_id=tenant_id,
                 container_type=container_type,
                 template_id=template_id,
                 authorized_users=authorized_users)
        )

        if self._post(Urls.container(*args, **kwargs), data=container.to_json()):
            if self.client.results.status_code == 200:
                response = CreateContainerResponse(self.client.to_dict())
                log.info(f'container created. {container} {response}')
                return response
        log.error(f'create container. {self.client.to_dict()}', enable_traceback=False)
        return False

    @staticmethod
    def base64_encode(data: bytes, **kwargs) -> str:
        encode = base64.b64encode(data, **kwargs)
        decode = encode.decode()
        return decode

    @_isConnected
    def create_container_attachment(
            self,
            container_id: int,
            file_name: str,
            file_content: bytes,
            metadata: dict = None, **kwargs) -> Optional[CreateContainerAttachmentResponse]:
        """Create container attachment"""

        if metadata:
            metadata = json.dumps(metadata)

        file_content = self.base64_encode(file_content)

        data = json.dumps(dict(
            container_id=container_id,
            file_name=file_name,
            file_content=file_content,
            metadata=metadata
        ))

        if self._post(Urls.container_attachment(**kwargs), data=data):
            response = CreateContainerAttachmentResponse(self.client.to_dict())
            log.info(f'create attachment: {response}')
            return response

        log.error(f'create attachment failed.', raise_exception=False)

    @_isConnected
    def create_vault(
            self,
            file_location,
            container_id=None,
            file_name=None,
            metadata=None,
            trace=False, **kwargs) -> Vault:
        """Add vault object"""

        data = Vault(dict(
            container=container_id,
            file_location=file_location,
            file_name=file_name,
            metadata=metadata,
            trace=trace
        ))

        if self._post(Urls.vault_add(identifire=data.id, **kwargs), data=data.to_json()):
            response = Vault(self._content_dict())
            log.info(msg=f'add vault: {response}')
            return response

        log.error(msg=f'add vault failed.', raise_exception=False)

    @_isConnected
    def delete_container(self, container_id, *args, **kwargs):
        """Delete containers"""
        assert isinstance(container_id, int)

        if self._delete(Urls.container(identifier=container_id, *args, **kwargs)):
            if self.client.results.status_code == 200:
                log.info(f'container deleted: {container_id}')
                return True
        log.error(f'delete container: {container_id}. {self.client.to_dict()}', enable_traceback=False)
        return False

    def isConnected(self) -> bool:
        """check if client can connect"""
        if self.config.isReady():
            if self._get(Urls.container(page_size=1)):
                log.info(f'client connected '
                         f'{self.config.host} '
                         f'[{self.client.results.status_code}] ')
                return True

        else:
            log.warn(f'client not connected')
        return False

    @_isConnected
    def filter_vault(self, filter: str, page_size: int = None, **kwargs) -> [Vault]:
        """Filter for matching vault files"""
        matches = []

        for sublist in self.list_vault_generator(page_size=page_size, **kwargs):
            for vault in sublist:
                if filter in vault.meta.values():
                    matches.append(vault)
                elif filter in vault.names:
                    matches.append(vault)
                elif filter in vault.__dict__.values():
                    matches.append(vault)

        return matches

    @_isConnected
    def generic_delete(self, api: str, **kwargs) -> Optional[GenericResponse]:
        """Make generic delete calls"""
        if self._delete(Urls.generic(api=api, **kwargs)):
            response = GenericResponse(self._content_dict())
            log.info(f'generic delete {api}: {response}')
            return response

        log.error(f'failed generic delete {api}', raise_exception=False)

    @_isConnected
    def generic_get(self, api: str, **kwargs) -> Optional[GenericResponse]:
        """Make generic get calls"""
        if self._get(Urls.generic(api=api, **kwargs)):
            response = GenericResponse(self._content_dict())
            log.info(f'generic get {api}: {response}')
            return response

        log.error(f'failed generic get {api}', raise_exception=False)

    @_isConnected
    def generic_post(self, api: str, data: dict, **kwargs) -> Optional[GenericResponse]:
        """Make generic post calls"""
        if self._post(Urls.generic(api=api, **kwargs), data=data):
            response = GenericResponse(self._content_dict())
            log.info(f'generic post {api}: {response}')
            return response

        log.error(f'failed generic post {api}', raise_exception=False)

    @_isConnected
    def get_artifact(self, artifact_id: int = None, **kwargs) -> Artifact:
        """Get artifact"""
        if self._get(Urls.artifact(identifier=artifact_id, **kwargs)):
            artifact = Artifact(self._content_dict())
            log.info(f'get artifact: {artifact}')
            return artifact

        log.error(f'artifact not found: {artifact_id}', enable_traceback=False)
        return Artifact()

    @_isConnected
    def get_container(self, container_id: int = None, **kwargs) -> Container:
        """Get container"""
        if self._get(Urls.container(identifier=container_id, **kwargs)):
            container = Container(self._content_dict())
            log.info(f'get container: {container}')
            return container

        log.error(f'container not found: {container_id}', enable_traceback=False)
        return Container()

    @_isConnected
    def get_playbook_run(self, playbook_run_id: str, **kwargs) -> Optional[PlaybookRun]:
        """Get running playbook"""
        if self._get(Urls.playbook_run(identifier=playbook_run_id, **kwargs)):
            response = PlaybookRun(self._content_dict())

            if response.status != 'failed':
                log.info(f'playbook run: {response}')
                return response

            log.error(f'playbook run failed: {response.message_to_dict}', enable_traceback=False)
            return response

        log.error(f'playbook failed: {self.client.errors}', enable_traceback=False)

    @_isConnected
    def get_vault(self, vault_id: int, **kwargs) -> Optional[Vault]:
        """Get vault object"""
        if self._get(Urls.vault(identifier=vault_id, **kwargs)):
            if self.client.results.status_code == 200:
                response = Vault(self._content_dict())
                log.info(msg=f'get vault: {response}')
                return response

        log.error(msg=f'get vault failed: {self.client.to_dict()}', raise_exception=False)

    @_isConnected
    def list_artifact(self, **kwargs) -> Response:
        """list artifacts"""
        if self._get(Urls.artifact(**kwargs)):
            response = Response(self._content_dict())
            log.info(f'list artifacts: {response.count}')
            return response

        return Response()

    @_isConnected
    def list_action_run(self, **kwargs) -> bool:
        """list action run"""
        if self._get(Urls.action_run(**kwargs)):
            self.containers = self._content_dict()
            return True
        return False

    @_isConnected
    def list_app(self, **kwargs) -> bool:
        """list app"""
        if self._get(Urls.app(**kwargs)):
            self.app = self._content_dict()
            return True
        return False

    @_isConnected
    def list_app_run(
            self,
            page: int = None,
            page_size: int = None, **kwargs) -> bool:
        """list app run"""
        if self._get(Urls.app_run(page=page, page_size=page_size, **kwargs)):
            self.app_run = AppRunResponse(self._content_dict())
            response = AppRunResponse(self._content_dict())
            response.data = [AppRunResults(x) for x in response.data]
            log.info(f'list app runs, page: {page} page_size: {page_size}, {response.summary()}')
            self.app_run = response
            return response
        return False

    @_isConnected
    def list_app_run_generator(
            self,
            page: int = 0,
            page_size: int = None,
            max_pages: int = None, **kwargs) -> AppRunResults or bool:
        """Generator for paging through app runs"""

        page = page

        while True:
            response = self.list_app_run(page=page, page_size=page_size, **kwargs)
            if response.data:
                app_runs = response.data
                num_pages = response.num_pages
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')

                if page >= num_pages or page >= max_pages:
                    log.info(f'list app runs finished')
                    return True

                yield app_runs
                page += 1

            elif response.data == []:
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')
                log.info(f'list app runs finished. {response}')
                return True

            elif response.data is None:
                log.error(f'list app runs failed', enable_traceback=True)
                return False

            else:
                log.info(f'no app runs. {response}')
                return True

        return False

    @_isConnected
    def list_app_run_by_playbook_run(
            self,
            playbook_run: int,
            page: int = None,
            page_size: int = None, **kwargs) -> bool:
        """list app run by playbook run"""

        app_runs = []

        for app_run in self.list_app_run_generator(page=page, page_size=page_size, **kwargs):
            if app_run.playbook_run == playbook_run:
                app_runs.append(app_run)

        return app_runs

    @_isConnected
    def list_artifacts(
            self,
            page: int = None,
            page_size: int = 1000, **kwargs) -> Response:
        """list artifacts"""
        if self._get(Urls.artifact(page=page, page_size=page_size, **kwargs)):
            response = Response(self._content())
            log.info(f'list artifacts: {len(response.data)}')
            return response
        return Response()

    @_isConnected
    def list_artifact_generator(
            self,
            page: int = 0,
            page_size: int = None,
            max_pages: int = None, **kwargs) -> [Container]:
        """Generator for paging through artifacts"""

        page = page

        while True:
            response = self.list_containers(page=page, page_size=page_size, **kwargs)
            if response.data:
                containers = [Container(x) for x in response.data]
                num_pages = response.num_pages
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')

                if page >= num_pages or page >= max_pages:
                    log.info(f'list container finished')
                    return True

                yield containers
                page += 1

            elif response.data == []:
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')
                log.info(f'list container finished. {response}')
                return True

            elif response.data is None:
                log.error(f'list container failed', enable_traceback=True)
                return False

            else:
                log.info(f'no containers. {response}')
                return True

        return False

    @_isConnected
    def list_asset(self, **kwargs) -> Response:
        """list asset"""
        if self._get(Urls.asset(**kwargs)):
            response = Response(self._content_dict())
            log.info(f'list assets: {len(response.data)}')
            return response
        return Response()

    @_isConnected
    def list_containers(
            self,
            page: int = None,
            page_size: int = 1000,
            *args, **kwargs) -> Response:
        """list containers"""

        url = Urls.container(page=page, page_size=page_size, *args, **kwargs)
        if self._get(url):
            response = Response(self._content_dict())
            log.info(f'list containers: {len(response.data)}')
            return response
        log.error(f'no containers', enable_traceback=False)
        return Response()

    @_isConnected
    def list_containers_generator(
            self,
            page: int = 0,
            page_size: int = None,
            max_pages: int = None, **kwargs) -> [Container]:
        """Generator for paging through containers"""

        page = page
        i = 0

        while True:
            if max_pages and i > max_pages:
                break

            response = self.list_containers(
                page=page,
                page_size=page_size, **kwargs
            )

            i += 1
            if response.data:
                containers = [Container(x) for x in response.data]
                num_pages = response.num_pages
                log.info(f'container page {page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')

                if page > num_pages:
                    log.info(f'list container finished')
                    break

                yield containers
                page += 1

            elif response.data == []:
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')
                log.info(f'list container finished. {response}')
                break

            elif response.data is None:
                log.error(f'list container failed', enable_traceback=True)
                break

            else:
                log.info(f'no containers. {response}')
                break

        return []

    @_isConnected
    def list_cluster_node(self, **kwargs) -> Optional[dict]:
        """List cluster node"""
        if self._get(Urls.cluster_node(**kwargs)):
            cluster_node = self._content_dict()
            return cluster_node

    @_isConnected
    def list_vault(self, **kwargs) -> Optional[VaultResponse]:
        """List vault"""
        if self._get(Urls.vault(**kwargs)):
            response = VaultResponse(self._content_dict())
            log.info(msg=f'list vault: {response}')
            return response

        log.error(msg=f'list vault failed.', raise_exception=False)

    @_isConnected
    def list_vault_generator(
            self,
            page: int = 0,
            page_size: int = None,
            max_pages: int = None, **kwargs) -> [Vault]:
        """Generator for paging through vaults"""
        i = 0

        while True:
            if max_pages and i >= max_pages:
                break

            response = self.list_vault(
                page=page,
                page_size=page_size, **kwargs
            )

            i += 1
            if response.data:
                vaults = [Vault(x) for x in response.data]
                num_pages = response.num_pages
                log.info(f'vault page {page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')

                if page > num_pages:
                    log.info(f'list vault finished')
                    break

                yield vaults
                page += 1

            elif response.data == []:
                log.info(f'{page}/{num_pages} ({round(page / num_pages * 100, 2)}%)')
                log.info(f'list vault finished. {response}')
                break

            elif response.data is None:
                log.error(f'list vault failed', enable_traceback=True)
                break

            else:
                log.info(f'no vaults. {response}')
                break

        return []

    @_isConnected
    def update_playbook(
            self,
            playbook_id: int = None,
            active: bool = None,
            cancel_runs: bool = False,
            **kwargs) -> Optional[UpdatePlaybookResponse]:
        """Update playbook active state"""
        data = dict(
            active=active,
            cancel_runs=cancel_runs
        )
        data = json.dumps(data)
        if self._post(Urls.playbook(identifier=playbook_id, **kwargs), data=data):
            if self.client.results.status_code == 200:
                response = UpdatePlaybookResponse(self._content_dict())
                log.info(f'update playbook: {data}')
                return response

        log.error(f'update failed: {self.client.to_dict()}', enable_traceback=False)

    @_isConnected
    def run_playbook(
            self,
            container_id: int,
            playbook_id: int,
            scope: str = 'all',
            run: bool = True,
            **kwargs) -> Optional[RunPlaybookResponse]:
        """Run playbook on a container"""
        data = dict(
            container_id=container_id,
            playbook_id=playbook_id,
            scope=scope,
            run=run
        )
        data = json.dumps(data)
        if self._post(Urls.playbook_run(**kwargs), data=data):
            if self.client.results.status_code == 200:
                response = RunPlaybookResponse(self._content_dict())
                log.info(f'run playbook: {data}')
                return response

        log.error(f'run failed: {self.client.to_dict()}', enable_traceback=False)



import unittest

from automon.helpers.regex import *


class RegexTest(unittest.TestCase):

    def test_magic(self):
        self.assertTrue(Magic)
        test = '100.15.96.234 helehleeajd'
        self.assertTrue(Magic.magic_box(test))

    def test_geolocation(self):
        self.assertTrue(geolocation)


if __name__ == '__main__':
    unittest.main()



import os
import tempfile
import functools

from urllib.parse import urlparse

from automon.log import Logging
from automon.helpers.dates import Dates
from automon.helpers.sanitation import Sanitation

from .config import SeleniumConfig
from .browser_types import BrowserType

log = Logging(name='SeleniumBrowser', level=Logging.DEBUG)


class SeleniumBrowser(object):

    def __init__(self, config: SeleniumConfig = None):
        self.config = config or SeleniumConfig()
        self.type = self._browser_type = BrowserType(self.config)
        self.driver = 'not set'
        self.window_size = ''

        self.url = ''
        self.status = ''

    def __repr__(self):
        if self.url:
            return f'{self.browser.name} {self.status} {self.url} {self.window_size}'
        return f'{self.browser}'

    @property
    def browser(self) -> BrowserType:
        return self.driver

    @property
    def get_log(self, log_type: str = 'browser') -> list:
        return self.browser.get_log(log_type)

    def _isRunning(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.browser != 'not set':
                return func(self, *args, **kwargs)
            log.error(f'Browser is not set!', enable_traceback=False)
            return False

        return wrapped

    def _screenshot_name(self, prefix=None):
        """Generate a unique filename

        :param browser:
        :param prefix: prefix filename with a string
        :return:
        """
        title = self.browser.title
        url = self.browser.current_url
        hostname = urlparse(url).hostname

        hostname_ = Sanitation.ascii_numeric_only(hostname)
        title_ = Sanitation.ascii_numeric_only(title)
        timestamp = Dates.filename_timestamp()

        if prefix:
            prefix = Sanitation.safe_string(prefix)
            return f'{prefix}_{hostname_}_{title_}_{timestamp}.png'

        return f'{hostname_}_{title_}_{timestamp}.png'

    @_isRunning
    def close(self):
        log.info(f'Browser closed')
        self.browser.close()

    @_isRunning
    def get(self, url: str) -> bool:
        try:
            self.url = url
            self.browser.get(url)
            self.status = 'OK'
            return True
        except Exception as e:
            self.status = 'ERROR'
            log.error(f'Error getting {url}: {e}', enable_traceback=False)

        return False

    @_isRunning
    def get_page(self, *args, **kwargs):
        return self.get(*args, **kwargs)

    @_isRunning
    def get_screenshot_as_png(self):
        return self.browser.get_screenshot_as_png()

    @_isRunning
    def get_screenshot_as_base64(self):
        return self.browser.get_screenshot_as_base64()

    @_isRunning
    def save_screenshot(self, filename: str = None, prefix: str = None, folder: str = None):

        if not filename:
            filename = self._screenshot_name(prefix)

        if not folder:
            path = os.path.abspath(tempfile.gettempdir())
        else:
            path = os.path.abspath(folder)

        if not os.path.exists(path):
            os.makedirs(path)

        save = os.path.join(path, filename)

        log.info(f'Saving screenshot to: {save}')

        return self.browser.save_screenshot(save)

    @_isRunning
    def isRunning(self):
        return True

    def set_browser(self, browser: BrowserType):
        self.set_driver(driver=browser)

    def set_driver(self, driver: BrowserType):
        if driver:
            self.driver = driver

    @_isRunning
    def set_resolution(self, width=1920, height=1080, device_type=None):

        if device_type == 'pixel3':
            width = 1080
            height = 2160

        if device_type == 'web-small' or device_type == '800x600':
            width = 800
            height = 600

        if device_type == 'web-small-2' or device_type == '1024x768':
            width = 1024
            height = 768

        if device_type == 'web-small-3' or device_type == '1280x960':
            width = 1280
            height = 960

        if device_type == 'web-small-4' or device_type == '1280x1024':
            width = 1280
            height = 1024

        if device_type == 'web' or device_type == '1920x1080':
            width = 1920
            height = 1080

        if device_type == 'web-2' or device_type == '1600x1200':
            width = 1600
            height = 1200

        if device_type == 'web-3' or device_type == '1920x1200':
            width = 1920
            height = 1200

        if device_type == 'web-large' or device_type == '2560x1400':
            width = 2560
            height = 1400

        if device_type == 'web-long' or device_type == '1920x3080':
            width = 1920
            height = 3080

        if not width and not height:
            width = 1920
            height = 1080

        self.window_size = width, height
        self.browser.set_window_size(width, height)

    @_isRunning
    def quit(self):
        self.browser.close()
        self.browser.quit()
        self.browser.stop_client()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):

    # def test_list_containers(self):
    #     if c.isConnected():
    #         self.assertTrue(c.list_containers())
    #     else:
    #         self.assertFalse(c.list_containers())

    def test_list_containers_generator(self):
        if c.isConnected():
            containers = [x for x in c.list_containers_generator(page_size=100, max_pages=2)]
            self.assertTrue(containers)
        else:
            self.assertFalse(c.list_containers_generator())


if __name__ == '__main__':
    unittest.main()



import os

from automon.log import Logging


class VdsConfig(object):
    ldap_port = 636
    rest_port = 8090

    rest_path = 'adap'

    def __init__(self, protocol: str = None,
                 server: str = None,
                 port: int = None,
                 path: str = None,
                 user: str = None,
                 password: str = None,
                 basedn: str = None):
        """VDS configuration"""

        self.prot = protocol or 'https'
        self.server = server or os.getenv('VDS_SERVER')
        self.port = port or os.getenv('VDS_PORT') or self.rest_port
        self.path = path or self.rest_path

        self.user = user or os.getenv('VDS_BIND_USER')
        self.bind_user = self.user
        self.password = password or os.getenv('VDS_PASSWORD')
        self.basedn = basedn or os.getenv('VDS_BASE_DN')

        self.uri = f'{self.prot}://{self.server}:{self.port}/{self.path}'

        self._log = Logging(name=VdsConfig.__name__, level=Logging.ERROR)

        [self._log.debug(f'config: {x}') for x in self.__dict__.items()]

    def __repr__(self):
        return f'{self.prot}://{self.server}:{self.port} ({self.bind_user}) ({self.basedn})'



from automon.log import Logging

from .datatypes import AbstractDataType

log = Logging('Vault', level=Logging.CRITICAL)


class Vault(AbstractDataType):
    contains: list = None
    deleted: bool = None
    first_seen_time: str
    hash: str = None
    id: int = None
    meta: dict
    names: list = None
    size: int = None
    sources: list
    tags: list = None
    version: int

    def __repr__(self):
        if self.names:
            return f'{",".join(self.names)}'
        return f'{self.to_dict()}'

    @property
    def sha1(self):
        return self.hash



import selenium
import functools

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains

from automon import Logging
from automon.integrations.seleniumWrapper.browser import SeleniumBrowser

from automon.helpers.sleeper import Sleeper
from automon.integrations.minioWrapper import MinioClient

from .config import InstagramConfig
from .urls import Urls
from .xpaths import XPaths

log = Logging('InstagramClientBrowser', level=Logging.INFO)


class InstagramClientBrowser:

    def __init__(self, login: str = None, password: str = None, config: InstagramConfig = None):
        """Instagram Browser Client"""
        self.config = config or InstagramConfig(login=login, password=password)
        self.login = self.config.login
        self.browser = SeleniumBrowser()
        self.browser.set_browser(self.browser.type.chrome)

    def __repr__(self):
        return f'{self.__dict__}'

    def _isRunning(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.browser.isRunning():
                return func(self, *args, **kwargs)
            return False

        return wrapped

    @property
    def urls(self):
        return Urls()

    @property
    def xpaths(self):
        return XPaths()

    def _isAuthenticated(self):
        return

    def _get_page(self, account):
        """ Get page
        """
        log.debug('[_get_page] getting {}'.format(account))

        page = 'https://instagram.com/{}'.format(account)
        browser = self.authenticated_browser
        return browser.get(page)

    def _get_stories(self, account):
        """ Retrieve story
        """
        story = 'https://www.instagram.com/stories/{}/'.format(account)
        num_of_stories = 0

        log.debug('[get_stories] {}'.format(story))

        browser = self.authenticated_browser
        browser.get(story)
        browser.browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/' + account)

        if 'Page Not Found' in browser.browser.title:
            log.debug('[get_stories] no stories for {}'.format(account))
            return num_of_stories

        Sleeper.seconds('instagram', 2)

        while True:
            try:
                next_story(browser)

                title = browser.browser.title
                if title == 'Instagram':
                    log.debug(('[get_stories] {} end of stories'.format(account)))
                    raise Exception
                num_of_stories += 1
                browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/' + account)
                Sleeper.seconds('watch the story for a bit', 1)
                browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/' + account)
            except:
                # TODO: disable browser proxy when done
                log.debug('[get_stories] done: {}'.format(account))
                return num_of_stories

    def _next_story(self, authenticated_browser):
        """ Click next story button
        """

        xpaths = [
            '//*[@id="react-root"]/section/div/div/section/div[2]/div[1]/div/div/div[2]/div/div/button',
            '//*[@id="react-root"]/section/div/div/section/div[2]/button[2]'
        ]

        found_btn = False
        for xpath in xpaths:
            try:
                browser = authenticated_browser
                button = browser.browser.find_element_by_xpath(xpath)
                found_btn = True
                log.debug('[next_story] next story')
                return button.click()
            except:
                pass

        if not found_btn:
            # no more stories. exit
            log.debug('[_next_story] no more stories')
            raise Exception

    def run_stories(self, limit=None):
        """Run
        """

        log.debug('[login] {}'.format(self.login))

        self.authenticated_browser = self.authenticate()

        # if self.authenticated_browser:
        #
        #     count = 0
        #     if limit:
        #
        #         for account in self.following:
        #             hevlog.logging.debug(
        #                 '[runrun] [{}] {} session: {}'.format(self.authenticated_browser.browser.name,
        #                                                       self.authenticated_browser.browser.title,
        #                                                       self.authenticated_browser.browser.session_id))
        #
        #             num_of_stories = get_stories(self.authenticated_browser, account)
        #
        #             hevlog.logging.info('[{}] {} stories'.format(account, num_of_stories))
        #
        #             count = count + 1
        #             if count == limit:
        #                 return
        #
        #     Sleeper.hour('instagram')
        #     self.run_stories()

    @_isRunning
    def authenticate(self):
        """Authenticate to Instagram
        """

        # TODO: create capture proxy
        #       send traffic to /api

        browser = self.browser
        actions = ActionChains(browser.browser)

        browser.set_resolution(1024, 1024)
        browser.get(self.urls.login_page)

        log.debug('[authenticate] {}'.format(self.urls.login_page))

        Sleeper.seconds('instagram get page', 1)

        actions.send_keys(Keys.TAB)
        actions.send_keys(self.login)
        actions.perform()

        Sleeper.seconds('instagram get page', 1)

        # the password field is sometimes div[3] and div[4]
        login_pass_xpaths = self.xpaths.login_pass_xpaths
        login_btn_xpaths = self.xpaths.login_btn_xpaths

        found_pass = False
        for xpath in login_pass_xpaths:
            try:
                login_pass = browser.browser.find_element_by_xpath(xpath)
                found_pass = True
                break
            except:
                pass

        Sleeper.seconds('instagram get page', 2)

        found_btn = False
        for xpath in login_btn_xpaths:
            try:
                login_btn = browser.browser.find_element_by_xpath(xpath)
                found_btn = True
                break
            except:
                pass

        if found_pass and found_btn:
            pass
        else:
            log.error('[browser] Authentication failed')
            log.debug('[browser] Found password field: {} Found login button: {}'.format(browser.browser.name,
                                                                                         found_pass,
                                                                                         found_btn))
            Sleeper.minute("instagram can't authenticate")
            return False

        login_pass.send_keys(self.password)
        login_btn.click()

        Sleeper.seconds('wait for instagram to log in', 5)

        log.debug(
            '[authenticated browser] [{}] {} session: {}'.format(browser.browser.name, browser.browser.title,
                                                                 browser.browser.session_id))

        browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/')

        return self.browser

    @_isAuthenticated
    def isAuthenticated(self):
        return


def authenticate(username, password, minio_client=None, retries=None):
    """Authenticates through browser and returns browser driver

    :param username: username string
    :param password: password string
    :param minio_client: minio client
    :param retries: not implemented
    :return: authenticated browser
    """

    while True:

        # TODO: create capture proxy
        #       send traffic to /api
        login_page = 'https://www.instagram.com/accounts/login/?source=auth_switcher'

        # browser = SeleniumBrowser(chrome())
        # browser = SeleniumBrowser(chrome_headless_nosandbox())
        browser = SeleniumBrowser(chrome_for_docker())
        # browser = SeleniumBrowser(chrome_sandboxed())
        # browser = SeleniumBrowser(chrome_headless_sandboxed())
        # browser = SeleniumBrowser(chrome_remote())

        browser.set_resolution('1024x768')

        if minio_client:
            browser.set_minio_client(minio_client)

        browser.get(login_page)

        log.debug('[authenticate] {}'.format(login_page))

        Sleeper.seconds('instagram get page', 1)

        browser.type(selenium.webdriver.common.keys.Keys.TAB)
        browser.type(username)

        Sleeper.seconds('instagram get page', 1)

        # the password field is sometimes div[3] and div[4]
        login_pass_xpaths = [
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[3]/div/label/input',
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[4]/div/label/input'
        ]

        login_btn_xpaths = [
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[4]/button',
            '//*[@id="react-root"]/section/main/div/article/div/div[1]/div/form/div[6]/button'
        ]

        found_pass = False
        for xpath in login_pass_xpaths:
            try:
                login_pass = browser.browser.find_element_by_xpath(xpath)
                found_pass = True
                break
            except:
                pass

        Sleeper.seconds('instagram get page', 2)

        found_btn = False
        for xpath in login_btn_xpaths:
            try:
                login_btn = browser.browser.find_element_by_xpath(xpath)
                found_btn = True
                break
            except:
                pass

        if found_pass and found_btn:
            break
        else:
            log.error('[browser] Authentication failed')

            log.debug(
                '[browser] Found password field: {} Found login button: {}'.format(browser.browser.name,
                                                                                   found_pass,
                                                                                   found_btn))

            Sleeper.minute("instagram can't authenticate")

    login_pass.send_keys(password)
    login_btn.click()

    Sleeper.seconds('wait for instagram to log in', 5)

    log.debug(
        '[authenticated browser] [{}] {} session: {}'.format(browser.browser.name, browser.browser.title,
                                                             browser.browser.session_id))
    browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/')

    return browser


def get_stories(authenticated_browser, account):
    """ Retrieve story
    """
    story = 'https://www.instagram.com/stories/{}/'.format(account)
    num_of_stories = 0
    # TODO: set browser to redirect to proxy here
    # TODO: check if account exists
    browser = authenticated_browser

    log.debug('[get_stories] {}'.format(story))

    browser.get(story)
    browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/' + account)

    if 'Page Not Found' in browser.browser.title:
        log.debug('[get_stories] no stories for {}'.format(account))
        return num_of_stories

    Sleeper.seconds('instagram', 2)

    while True:
        try:
            next_story(browser)

            title = browser.browser.title
            if title == 'Instagram':
                log.debug(('[get_stories] {} end of stories'.format(account)))
                raise Exception
            num_of_stories += 1
            browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/' + account)
            Sleeper.seconds('watch the story for a bit', 1)
            browser.save_screenshot_to_minio(bucket_name='screenshots', prefix='instagram/' + account)
        except:
            # TODO: disable browser proxy when done
            log.debug('[get_stories] done: {}'.format(account))
            return num_of_stories


def next_story(authenticated_browser):
    """ Click next story button
    """

    xpaths = [
        '//*[@id="react-root"]/section/div/div/section/div[2]/div[1]/div/div/div[2]/div/div/button',
        '//*[@id="react-root"]/section/div/div/section/div[2]/button[2]'
    ]

    found_btn = False
    for xpath in xpaths:
        try:
            browser = authenticated_browser
            button = browser.browser.find_element_by_xpath(xpath)
            found_btn = True
            log.debug('[next_story] next story')
            return button.click()
        except:
            pass

    if not found_btn:
        # no more stories. exit
        log.debug('[next_story] no more stories')
        raise Exception


def get_page(authenticated_browser, account):
    """ Get page
    """
    # TODO: need to download page
    log.debug('[get_page] getting {}'.format(account))
    page = 'https://instagram.com/{}'.format(account)
    browser = authenticated_browser
    return browser.get(page)


def runrun(browser, account):
    log.debug(
        '[runrun] [{}] {} session: {}'.format(browser.browser.name, browser.browser.title,
                                              browser.browser.session_id))

    num_of_stories = get_stories(browser, account)

    log.info('[{}] {} stories'.format(account, num_of_stories))

    # Sleeper.minute('instagram')

    return True


def test_run(config):
    client = MinioClient(config['minio-hev'], secure=False)

    instagram_config = config['instagram']
    login = instagram_config['login']['account']
    password = instagram_config['login']['password']
    accounts = instagram_config['following']

    log.debug('[login] {}'.format(login))
    log.info('Running...')
    log.info('[accounts] {}'.format(len(accounts)))

    while True:

        if len(accounts) > 0:

            browser = authenticate(login, password, client)

            for account in accounts:

                while True:
                    if runrun(browser, account):
                        break
                    else:
                        browser = authenticate(login, password, client)

                    break
                break
            break
